@@ -1,13 +1,17 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2003, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
- *  This module contains all 3D drawing related stuff, like coordinate
+ *  This module contains almost all 3D drawing related stuff, like coordinate
  *  transformation and drawing routines based on generic low-level graphics
  *  routines, as provided by the Outdev-struct pointed to by *out.
+ *  The only 3D drawing code not included here, is the code for drawing the
+ *  the gain pattern as an opaque surface, i.e., with hidden-line removal;
+ *  that code is in draw_opaque.c, because of its size and complexity.
+ *
  *  Note: the code for drawing the 2D plots of several quantities versus
  *  frequency is in 'freqplot.c'.
  *  
@@ -16,13 +20,12 @@
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
-#include <X11/X.h>
-#include <X11/Xlib.h>
+#include <string.h>
+#include <gdk/gdk.h>
 
 #include "xnecview.h"
 
-
-/* ------------------------- coordinate conversion, drawing routines -------------------- */
+/* ------------------------- coordinate conversion and related arithmetic -------------------------------------------------- */
 
                                /* these define the projection: */
 double phi=ini_phi,theta=ini_theta;    /* angles defining direction of eye */
@@ -77,6 +80,7 @@ void proj(Point *p,double *c)      /* perform the projection: *p defines a point
 }
 
 
+
 void interpolate(double *p,double f,double *a,double *b)   /* interpolate linearly between two points in the window (f=0..1 -> *p=*a..*b) */
 {
    p[0]=(1-f)*a[0]+f*b[0];
@@ -88,36 +92,7 @@ void interpolate(double *p,double f,double *a,double *b)   /* interpolate linear
 /* test whether a line through two points is "ascending" ( / ) as opposed to "descending" ( \ ) */
 #define ascending(c1,c2) (((c2)[0]-(c1)[0])*((c2)[1]-(c1)[1])<0)
 
-void draw_axes()		/* draw the axes */
-{
-   double c0[2],c1[2],c2[2],c3[2];
-   Point pp;
-
-   SetForeground(&c_axis);
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
-   pp.x=0; pp.y=0; pp.z=0; proj(&pp,c0);
-   pp.x=Axislen*extr; pp.y=0; pp.z=0; proj(&pp,c1);
-   pp.x=0; pp.y=Axislen*extr; pp.z=0; proj(&pp,c2);
-   pp.x=0; pp.y=0; pp.z=Axislen*extr; proj(&pp,c3);
-   DrawLine(c0[0], c0[1], c1[0], c1[1]);
-   DrawLine(c0[0], c0[1], c2[0], c2[1]);
-   DrawLine(c0[0], c0[1], c3[0], c3[1]);
-   if (!quick) {
-      if (c1[0]>=c0[0] || !ascending(c0,c1)) DrawStringLL(c1[0]+2,c1[1],"X"); else DrawStringUL(c1[0]+2,c1[1],"X");
-      if (c2[0]>=c0[0] || !ascending(c0,c2)) DrawStringLL(c2[0]+2,c2[1],"Y"); else DrawStringUL(c2[0]+2,c2[1],"Y");
-      if (c3[0]>=c0[0] || !ascending(c0,c3)) DrawStringLL(c3[0]+2,c3[1],"Z"); else DrawStringUL(c3[0]+2,c3[1],"Z");
-   }
-
-   SetLineAttributes(0, LineOnOffDash, CapButt, JoinRound);
-   pp.x=0; pp.y=0; pp.z=0; proj(&pp,c0);
-   pp.x=-Axislen*extr; pp.y=0; pp.z=0; proj(&pp,c1);
-   DrawLine(c0[0], c0[1], c1[0], c1[1]);
-   pp.x=0; pp.y=-Axislen*extr; pp.z=0; proj(&pp,c1);
-   DrawLine(c0[0], c0[1], c1[0], c1[1]);
-   pp.x=0; pp.y=0; pp.z=-Axislen*extr; proj(&pp,c1);
-   DrawLine(c0[0], c0[1], c1[0], c1[1]);
-}
-
+/* ------------------------- antenna structure drawing ------------------------------------------------------------------- */
 
 void draw_wires(void)                /* draw the wires of the antenna */
 {
@@ -128,7 +103,7 @@ void draw_wires(void)                /* draw the wires of the antenna */
    if (numwires==0) return;
 
    SetForeground(&c_wire);
-   SetLineAttributes(2, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(2, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    for (wi=wires;wi<wires+numwires;wi++) {
       proj(&wi->p0,c0);
       proj(&wi->p1,c1);
@@ -151,7 +126,7 @@ void draw_loads(void)                 /* (re)draw loaded wires */
 
    if (numloads==0) return;
    SetForeground(&c_load);
-   SetLineAttributes(3, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(3, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    for (lo=loads;lo<loads+numloads;lo++) {
       proj(&lo->wire->p0,c0);
       proj(&lo->wire->p1,c1);
@@ -187,7 +162,7 @@ void draw_excis(void)                 /* (re)draw wires with excitations */
 
    if (numexcis==0) return;
    SetForeground(&c_exci);
-   SetLineAttributes(4, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(4, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    for (ex=excis;ex<excis+numexcis;ex++) {
       proj_segment(ex->wire, ex->seg, c0, c1);
       DrawLine(c0[0], c0[1], c1[0], c1[1]);
@@ -206,18 +181,18 @@ void draw_netws(void)                 /* (re)draw networks and transmission line
       proj_segment(ne->wire1, ne->seg1, c10, c11);
       proj_segment(ne->wire2, ne->seg2, c20, c21);
       if (ne->type==1) {
-         SetLineAttributes(1, LineSolid, CapButt, JoinRound);
+         SetLineAttributes(1, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
          DrawLine(c10[0], c10[1], c20[0], c20[1]);
          DrawLine(c11[0], c11[1], c21[0], c21[1]);
       } else if (ne->type==-1) {
-         SetLineAttributes(1, LineSolid, CapButt, JoinRound);
+         SetLineAttributes(1, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
          DrawLine(c10[0], c10[1], c21[0], c21[1]);
          DrawLine(c11[0], c11[1], c20[0], c20[1]);
       } else {
          double d1[2],d2[2];
          interpolate(d1,0.5,c10,c11);
          interpolate(d2,0.5,c20,c21);
-         SetLineAttributes(2, LineSolid, CapButt, JoinRound);
+         SetLineAttributes(2, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
          DrawLine(d1[0], d1[1], d2[0], d2[1]);
       }
    }
@@ -231,7 +206,7 @@ void draw_surfaces(void)
    double ca[2],cc[2];
 
    if (numsurfaces==0) return;
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    for (su=surfaces;su<surfaces+numsurfaces;su++) {
       if ( eyevec.x * (su->pa.x-su->pc.x) + eyevec.y * (su->pa.y-su->pc.y) + eyevec.z * (su->pa.z-su->pc.z) >= 0 )
          SetForeground(&c_surf);
@@ -280,6 +255,8 @@ void draw_antenna(int ie)              /* draw the entire antenna */
 }
 
 
+/* ------------------------- currents/charges drawing -------------------------------------------------------------------- */
+
 int cmpsegdist(const Segcur **s1, const Segcur **s2)
 {
    double d;
@@ -290,7 +267,7 @@ int cmpsegdist(const Segcur **s1, const Segcur **s2)
 }
 
 
-void addvector(Point *pe,double a[3],double q)
+void addvector(Point *pe,float a[3],double q)
 {
    pe->x+=q*a[0];
    pe->y+=q*a[1];
@@ -328,11 +305,11 @@ void draw_currents_anim(int ie,Currents *cu)      /* draw the currents and charg
       pe.x = se->c.x + sc*ev[0]; pe.y = se->c.y + sc*ev[1]; pe.z = se->c.z + sc*ev[2];
       proj(&se->c,c0);
       proj(&pe,c1);
-      SetLineAttributes(2, LineSolid, CapButt, JoinRound);
+      SetLineAttributes(2, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
       SetForeground(&c_wire);
       DrawLine(c0[0], c0[1], c1[0], c1[1]);
 
-      SetLineAttributes(1, LineSolid, CapButt, JoinRound);
+      SetLineAttributes(1, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
       if (q<0) SetForeground(&c_qneg); else SetForeground(&c_qpos);
       q=fabs(q);
       pe=se->c;
@@ -389,6 +366,7 @@ void draw_currents_noanim(int ie,Currents *cu)      /* draw the currents distrib
          phase=se->phase;
          switch (polarization) {
             case POLnone:
+            case POLcolour:
                a=se->a;
                break;
             case POLhor:
@@ -414,10 +392,10 @@ void draw_currents_noanim(int ie,Currents *cu)      /* draw the currents distrib
       }
 
       if (!quick && maxthickness*a>=0.5) {
-         SetLineAttributes(maxthickness*a+0.5, LineSolid, CapButt, JoinRound);
+         SetLineAttributes(maxthickness*a+0.5, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
          SetForeground(c_currents+(int)(phase*NC_phase/360));
       } else {
-         SetLineAttributes(1, LineSolid, CapButt, JoinRound);
+         SetLineAttributes(1, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
          SetForeground(&c_inactive);
       }
       DrawLine(c0[0], c0[1], c1[0], c1[1]);
@@ -447,7 +425,7 @@ void draw_phasecircle(void)
 
    r=winsize/10;
    xc=yc=r+r/2+1;
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    step = 1/(2*M_PI*r) * 360;
 
    for (i=0;i<NC_phase/4;i++) {
@@ -459,7 +437,7 @@ void draw_phasecircle(void)
    j=3*(int)(2*M_PI*r/NC_phase+2);
    do {
       j/=3;
-      SetLineAttributes(j, LineSolid, CapButt, JoinRound);
+      SetLineAttributes(j, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
       for (i=0;i<NC_phase/4;i++) {
          rs=r*sintab[i];
          rc=r*costab[i];
@@ -484,6 +462,7 @@ void draw_phasecircle(void)
 }
 
 
+/* ------------------------- near field drawing -------------------------------------------------------------------------- */
 
 void draw_nearfield(int ie,NECoutput *ne)      /* draw the E and H field and Poynting vectors */
 {
@@ -507,7 +486,7 @@ void draw_nearfield(int ie,NECoutput *ne)      /* draw the E and H field and Poy
    sch=f*hscale/ne->maxh;
    scp=2*f*escale*hscale/(ne->maxe*ne->maxh);
 
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
    si = sin(-animphase);
    co = cos(-animphase);
    while (nf) {
@@ -553,22 +532,50 @@ void draw_nearfield(int ie,NECoutput *ne)      /* draw the E and H field and Poy
 
 
 
+/* ------------------------- gain pattern drawing (non-opaque) ----------------------------------------------------------- */
 
 
+void draw_polref(void)
+{
+   SetForeground(&c_gain_lin); DrawString(winsizex-5,5+0*fontheight,"linear",1,1);
+   SetForeground(&c_gain_lhcp); DrawString(winsizex-5,5+1*fontheight,"left-hand circular",1,1);
+   SetForeground(&c_gain_rhcp); DrawString(winsizex-5,5+2*fontheight,"right-hand circular",1,1);
+}
+
+
+void setpolcolour(double axial)
+{
+   if (axial>Polthr) SetForeground(&c_gain_lhcp);
+   else if (axial<-Polthr) SetForeground(&c_gain_rhcp);
+   else SetForeground(&c_gain_lin);
+}
 
 
 void draw_gain_theta(int i,Radpattern *rp)
 {
    int j;
    double c0[2],c1[2];
+   double a0,a1;
 
    proj(&rp->gpo[0][i],c1);
+   a1=rp->gain[0][i].axial;
    for (j=1;j<rp->numphi;j++) {
       c0[0]=c1[0]; c0[1]=c1[1];
       proj(&rp->gpo[j][i],c1);
+      if (polarization==POLcolour) {
+         a0=a1;
+         a1=rp->gain[j][i].axial;
+         setpolcolour((a0+a1)/2);
+      }
       DrawLine(c0[0], c0[1], c1[0], c1[1]);
    }
-   proj(&rp->gpo[0][i],c0);
+   c0[0]=c1[0]; c0[1]=c1[1];
+   proj(&rp->gpo[0][i],c1);
+   if (polarization==POLcolour) {
+      a0=a1;
+      a1=rp->gain[0][i].axial;
+      setpolcolour((a0+a1)/2);
+   }
    DrawLine(c0[0], c0[1], c1[0], c1[1]);
 }
 
@@ -576,11 +583,19 @@ void draw_gain_phi(int j,Radpattern *rp)
 {
    int i;
    double c0[2],c1[2];
-   proj(&rp->gpo[j][0],c0);
+   double a0,a1;
+
+   proj(&rp->gpo[j][0],c1);
+   a1=rp->gain[j][0].axial;
    for (i=1;i<rp->numtheta;i++) {
-      proj(&rp->gpo[j][i],c1);
-      DrawLine(c0[0], c0[1], c1[0], c1[1]);
       c0[0]=c1[0]; c0[1]=c1[1];
+      proj(&rp->gpo[j][i],c1);
+      if (polarization==POLcolour) {
+         a0=a1;
+         a1=rp->gain[j][i].axial;
+         setpolcolour((a0+a1)/2);
+      }
+      DrawLine(c0[0], c0[1], c1[0], c1[1]);
    }
 }
 
@@ -590,15 +605,34 @@ void draw_gain_phi(int j,Radpattern *rp)
 #define R270 (1.5*M_PI)
 #define R360 (2*M_PI)
 
+#include <sys/timeb.h>
+
 void draw_gain(int ie,Radpattern *rp)
 {
    int i,j;
+   int gainplot2=gainplot;
 
    SetForeground(&c_gain);
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
+
+   if (gainplot==GPopaque && !quick) {
+      Radpattern *p;
+      int ok=0;
+      p=rp;
+      while (p) {
+         if (!opaque_impossible(p)) { 
+            draw_opaque(p);
+            ok=1;
+         }
+         p=p->next;
+      }
+      if (ok) return;
+      fprintf(stderr,"Opaque gain plot not possible; reason: %s.\n",opaque_impossible(rp));
+      gainplot2=GPframe;
+   }
 
    while (rp) {
-      if (gainplot==GP3d && !quick) {
+      if (gainplot2==GPframe && !quick) {
 
          for (i=0;i<rp->numtheta;i++) draw_gain_theta(i,rp);
          if (Pending() && ie) return;
@@ -657,7 +691,7 @@ void draw_gainscale(int ie)
    if (!scaleplot) return;
 
    SetForeground(&c_scale);
-   SetLineAttributes(0, LineSolid, CapButt, JoinRound);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
 
    if (firsttime) {
       for (i=Npt-1, a=0; i>=0; i--, a+=da) {
@@ -758,33 +792,79 @@ void draw_gainscale(int ie)
 }
 
 
+/* ------------------------- draw complete picture ------------------------------------------------------------------------- */
+
+void draw_axes()		/* draw the axes */
+{
+   double c0[2],c1[2],c2[2],c3[2];
+   Point pp;
+
+   SetForeground(&c_axis);
+   SetLineAttributes(0, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
+   pp.x=0; pp.y=0; pp.z=0; proj(&pp,c0);
+   pp.x=Axislen*extr; pp.y=0; pp.z=0; proj(&pp,c1);
+   pp.x=0; pp.y=Axislen*extr; pp.z=0; proj(&pp,c2);
+   pp.x=0; pp.y=0; pp.z=Axislen*extr; proj(&pp,c3);
+   DrawLine(c0[0], c0[1], c1[0], c1[1]);
+   DrawLine(c0[0], c0[1], c2[0], c2[1]);
+   DrawLine(c0[0], c0[1], c3[0], c3[1]);
+   if (!quick) {
+      if (c1[0]>=c0[0] || !ascending(c0,c1)) DrawStringLL(c1[0]+2,c1[1],"X"); else DrawStringUL(c1[0]+2,c1[1],"X");
+      if (c2[0]>=c0[0] || !ascending(c0,c2)) DrawStringLL(c2[0]+2,c2[1],"Y"); else DrawStringUL(c2[0]+2,c2[1],"Y");
+      if (c3[0]>=c0[0] || !ascending(c0,c3)) DrawStringLL(c3[0]+2,c3[1],"Z"); else DrawStringUL(c3[0]+2,c3[1],"Z");
+   }
+
+   SetLineAttributes(0, GDK_LINE_ON_OFF_DASH, GDK_CAP_ROUND, GDK_JOIN_ROUND);
+   pp.x=0; pp.y=0; pp.z=0; proj(&pp,c0);
+   pp.x=-Axislen*extr; pp.y=0; pp.z=0; proj(&pp,c1);
+   DrawLine(c0[0], c0[1], c1[0], c1[1]);
+   pp.x=0; pp.y=-Axislen*extr; pp.z=0; proj(&pp,c1);
+   DrawLine(c0[0], c0[1], c1[0], c1[1]);
+   pp.x=0; pp.y=0; pp.z=-Axislen*extr; proj(&pp,c1);
+   DrawLine(c0[0], c0[1], c1[0], c1[1]);
+}
+
+
 void draw_all(int ie)
 {
    char s[100];
    do {
       if (Pending() && ie) return;
       ClearWindow();
-      if (gainplot==GP2d || gainplot==GP3d) draw_gainscale(ie);
+      if (gainplot==GPslice || gainplot==GPframe || gainplot==GPopaque) draw_gainscale(ie);
       draw_axes();
       if (Pending() && ie) break;
       if (structplot) {
          if (structplot==SPcurrents) {
-            draw_currents_noanim(ie,neco[rp_index].cu);
-            if (!quick) draw_phasecircle();
+            if (rp_index>=0) {
+               draw_currents_noanim(ie,neco[rp_index].cu);
+               if (!quick) draw_phasecircle();
+            }
          } else if (structplot==SPanim) {
-            draw_currents_anim(ie,neco[rp_index].cu);
+            if (rp_index>=0) {
+               draw_currents_anim(ie,neco[rp_index].cu);
+            }
          }
          else draw_antenna(ie);
       }
       if (Pending() && ie) break;
-      if (gainplot==GP2d || gainplot==GP3d) draw_gain(ie,neco[rp_index].rp);
-      if (gainplot==GPnearfield) draw_nearfield(ie,neco+rp_index);
-      else if (rp_index>=0) {
+      if (rp_index>=0) {
+         if (gainplot==GPslice || gainplot==GPframe || gainplot==GPopaque) {
+            draw_gain(ie,neco[rp_index].rp);
+            if (polarization==POLcolour) draw_polref();
+         }
+         if (gainplot==GPnearfield) draw_nearfield(ie,neco+rp_index);
          if (neco[rp_index].rp) {
+            double maxgain,vgain;
+            if (polarization==POLnone || polarization==POLcolour) {
+               maxgain=neco[rp_index].d[neco_maxgain],
+               vgain=neco[rp_index].d[neco_vgain];
+            } else {
+               maxgain=neco[rp_index].d[Neco_polgain+Neco_gsize*polarization];
+               vgain=neco[rp_index].d[neco_vgain2];
+            }
             sprintf(s,"f = %g MHz   maxgain = %g dBi   vgain = %g dBi",
-               neco[rp_index].f,
-               neco[rp_index].d[neco_maxgain],
-               neco[rp_index].d[neco_vgain]);
+               neco[rp_index].f, maxgain, vgain);
          } else {
             sprintf(s,"f = %g MHz", neco[rp_index].f);
 
