@@ -1,9 +1,9 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2000, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2002, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
  *  This module parses the NEC input file (i.e., the file describing the
  *  antenna's structure).
@@ -31,7 +31,6 @@
 
 int nextabs;        /* next absolute segment number to be assigned */
 
-double GeomScale;   /* geometry scale factor, set by GS cards */
 
 
 
@@ -80,8 +79,8 @@ int reflect(int plane,int inc)           /* perform a reflection of all antenna
    Surface *s,*s0;
    int i;
 
+   EXPAND_IF_NECESSARY(numwires*2,maxwires,wires)
    for ( i=numwires, w0=wires, w=wires+numwires ; i>0 ; i--, w0++, w++, numwires++ ) {
-      if (numwires==MAXWIRES) return Err_nwires;
       *w=*w0;
       switch (plane) {
          case 0: w->p0.x=-w->p0.x; w->p1.x=-w->p1.x; break;
@@ -91,8 +90,8 @@ int reflect(int plane,int inc)           /* perform a reflection of all antenna
       if (w->itg>0) w->itg+=inc;
       w->abs=nextabs; nextabs+=w->ns;
    }
+   EXPAND_IF_NECESSARY(numsurfaces*2,maxsurfaces,surfaces)
    for ( i=numsurfaces, s0=surfaces, s=surfaces+numsurfaces ; i>0 ; i--, s0++, s++, numsurfaces++ ) {
-      if (numsurfaces==MAXSURFACES) return Err_nsurfaces;
       *s=*s0;
       switch (plane) {
          case 0: s->p0.x=-s->p0.x; s->p1.x=-s->p1.x; s->p2.x=-s->p2.x; s->p3.x=-s->p3.x; s->pc.x=-s->pc.x; s->pa.x=-s->pa.x; break;
@@ -151,7 +150,7 @@ int read_nec_EX(char *s)     /* EX -> excitation */
    if (sscanf(s,"EX%d%d%d",&type,&tag,&seg) != 3) return Err_scan;
    if (type!=0 && type!=5) { fprintf(stderr,"Only voltage sources can be displayed: %s\n",s); return Err_misc; }
 
-   if (numexcis==MAXEXCIS) return Err_nexcis;
+   EXPAND_IF_NECESSARY(numexcis,maxexcis,excis)
    err=find_segment(tag, seg, "EX", &excis[numexcis].wire, &excis[numexcis].seg);
    if (err) return err;
    numexcis++;
@@ -168,7 +167,7 @@ int read_nec_TL(char *s)     /* TL -> transmission line */
    removecommas(s);
    if (sscanf(s,"TL%d%d%d%d%lg",&tag1,&seg1,&tag2,&seg2,&imp) != 5) return Err_scan;
 
-   if (numnetws==MAXNETWS) return Err_nnetws;
+   EXPAND_IF_NECESSARY(numnetws,maxnetws,netws)
    err=find_segment(tag1, seg1, "TL", &netws[numnetws].wire1, &netws[numnetws].seg1);
    if (err) return err;
    err=find_segment(tag2, seg2, "TL", &netws[numnetws].wire2, &netws[numnetws].seg2);
@@ -187,7 +186,7 @@ int read_nec_NT(char *s)     /* NT -> network */
    removecommas(s);
    if (sscanf(s,"NT%d%d%d%d",&tag1,&seg1,&tag2,&seg2) != 4) return Err_scan;
 
-   if (numnetws==MAXNETWS) return Err_nnetws;
+   EXPAND_IF_NECESSARY(numnetws,maxnetws,netws)
    err=find_segment(tag1, seg1, "NT", &netws[numnetws].wire1, &netws[numnetws].seg1);
    if (err) return err;
    err=find_segment(tag2, seg2, "NT", &netws[numnetws].wire2, &netws[numnetws].seg2);
@@ -200,7 +199,7 @@ int read_nec_NT(char *s)     /* NT -> network */
 
 int addload(Wire *wi,int first,int last)   /* add one load */
 {
-   if (numloads==MAXLOADS) return Err_nloads;
+   EXPAND_IF_NECESSARY(numloads,maxloads,loads)
    loads[numloads].wire=wi;
    loads[numloads].firstseg=first;
    loads[numloads].lastseg=last;
@@ -282,9 +281,7 @@ int read_nec_GA(char *s)     /* GA -> wire arc */
 
    removecommas(s);
    if (sscanf(s,"GA%d%d%lg%lg%lg%lg",&itg,&ns,&rada,&ang1,&ang2,&rad) != 6) return Err_scan;
-   if (numwires+ns>MAXWIRES) return Err_nwires;
-   rada*=GeomScale;
-   rad*=GeomScale;
+   EXPAND_IF_NECESSARY(numwires+ns,maxwires,wires)
    phi=ang1*M_PI/180;
    incphi=(ang2-ang1)*M_PI/180/ns;
    po.x=cos(phi)*rada; po.y=0; po.z=sin(phi)*rada; updateextremes(&po);
@@ -318,15 +315,8 @@ int read_nec_GH(char *s)     /* GH -> helix */
 
    removecommas(s);
    if (sscanf(s,"GH%d%d%lg%lg%lg%lg%lg%lg%lg",&itg,&ns,&sp,&hl,&a1,&b1,&a2,&b2,&rad) != 9) return Err_scan;
-   if (numwires+ns>MAXWIRES) return Err_nwires;
+   EXPAND_IF_NECESSARY(numwires+ns,maxwires,wires)
    if (hl==0) { fprintf(stderr,"Helix with length=0 not yet supported.\n"); return Err_misc; }
-   sp*=GeomScale;
-   hl*=GeomScale;
-   rad*=GeomScale;
-   a1*=GeomScale;
-   a2*=GeomScale;
-   b1*=GeomScale;
-   b2*=GeomScale;
    z=0; incz=hl/ns;
    rx=a1; incrx=(a2-a1)/ns;
    ry=b1; incry=(b2-b1)/ns;
@@ -378,10 +368,6 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
    i=sscanf(s,"GM%d%d%lg%lg%lg%lg%lg%lg%d",&itsi,&nrpt,&rox,&roy,&roz,&xs,&ys,&zs,&its);
    if (i<2) return Err_scan;
 
-   xs*=GeomScale;
-   ys*=GeomScale;
-   zs*=GeomScale;
-
    rox*=M_PI/180.0; roy*=M_PI/180.0; roz*=M_PI/180.0;
 
    mat[0]=mat[4]=mat[8]=1;
@@ -415,6 +401,7 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
    mat[4] = si*mmat[1] + co*mmat[4];
    mat[5] = si*mmat[2] + co*mmat[5];
 
+   EXPAND_IF_NECESSARY(numwires*(nrpt+1),maxwires,wires)
    if (nrpt==0) {
       w00=w1=wires;
    } else {
@@ -428,7 +415,6 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
          i=0;
          do {
             if (nrpt>0) {
-               if (numwires==MAXWIRES) return Err_nwires;
                numwires++;
             } else {
                w1=w0;
@@ -453,11 +439,12 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
       s0=s1=surfaces;
       i=numsurfaces;
    } else {
-      s0=surfaces;
-      s1=surfaces+numsurfaces;
+      int oldnumsurfaces=numsurfaces;
       i=numsurfaces*nrpt;
       numsurfaces*=(1+nrpt);
-      if (numsurfaces>MAXSURFACES) return Err_nsurfaces;
+      EXPAND_IF_NECESSARY(numsurfaces,maxsurfaces,surfaces)
+      s0=surfaces;
+      s1=surfaces+oldnumsurfaces;
    }
    while (i--) {
       s1->type=s0->type;
@@ -487,11 +474,29 @@ int read_nec_GR(char *s)    /* GR -> generate cylindrical structure */
 int read_nec_GS(char *s)      /* GS -> geometry scale */
 {
    double a;
+   Wire *w;
+   Surface *su;
+   int i;
    removecommas(s);
    if (sscanf(s,"GS%*d%*d%lg",&a) != 1) return Err_scan;
-   GeomScale/=a;      /* note: in principle, a GS card scales all previously entered geometry; 
-                               however, since xnecview only needs relative sizes, we get the same
-                               picture by just inversely scaling all later geometry entries */
+   for (i=0,w=wires;i<numwires;i++,w++) {
+      w->p0.x*=a;
+      w->p0.y*=a;
+      w->p0.z*=a;
+      w->p1.x*=a;
+      w->p1.y*=a;
+      w->p1.z*=a;
+      w->rad*=a;
+   }
+   for (i=0,su=surfaces;i<numsurfaces;i++,su++) {
+      su->p0.x*=a; su->p0.y*=a; su->p0.z*=a;
+      su->p1.x*=a; su->p1.y*=a; su->p1.z*=a;
+      su->p2.x*=a; su->p2.y*=a; su->p2.z*=a;
+      su->p3.x*=a; su->p3.y*=a; su->p3.z*=a;
+      su->pc.x*=a; su->pc.y*=a; su->pc.z*=a;
+      su->pa.x*=a; su->pa.y*=a; su->pa.z*=a;
+   }
+   extr_str*=a;
    return 0;                               
 }
 
@@ -499,19 +504,12 @@ int read_nec_GW(char *s)    /* GW -> straight wire */
 {
    Wire *w;
    int i;
-   if (numwires==MAXWIRES) return Err_nwires;
+   EXPAND_IF_NECESSARY(numwires,maxwires,wires)
    w=wires+numwires++;
    removecommas(s);
    w->rad=0;
-   i=sscanf(s,"GW%d%d%lg%lg%lg%lg%lg%lg%lg",&w->itg,&w->ns,&w->p0.x,&w->p0.y,&w->p0.z,&w->p1.x,&w->p1.y,&w->p1.z,&w->rad);
+   i=sscanf(s,"GW%d%d%g%g%g%g%g%g%lg",&w->itg,&w->ns,&w->p0.x,&w->p0.y,&w->p0.z,&w->p1.x,&w->p1.y,&w->p1.z,&w->rad);
    if (i!=8 && i!=9) return Err_scan;  /* note: omitting the radius seems to be allowed, for tapered wires */
-   w->p0.x*=GeomScale;
-   w->p0.y*=GeomScale;
-   w->p0.z*=GeomScale;
-   w->p1.x*=GeomScale;
-   w->p1.y*=GeomScale;
-   w->p1.z*=GeomScale;
-   w->rad*=GeomScale;
    w->abs=nextabs; nextabs+=w->ns;
    w->dispnr=1;
    updateextremes(&w->p0);
@@ -534,12 +532,10 @@ int read_nec_SC(char *s)     /* SC -> continuation of SM */
 {
    Surface *su;
    Point p3;
+   EXPAND_IF_NECESSARY(numsurfaces,maxsurfaces,surfaces)
    su=surfaces+numsurfaces++;
    removecommas(s);
-   if (sscanf(s,"SC%*i%*i%lg%lg%lg",&su->p2.x,&su->p2.y,&su->p2.z) != 3) return Err_scan;
-   su->p2.x*=GeomScale;
-   su->p2.y*=GeomScale;
-   su->p2.z*=GeomScale;
+   if (sscanf(s,"SC%*i%*i%g%g%g",&su->p2.x,&su->p2.y,&su->p2.z) != 3) return Err_scan;
    updateextremes(&su->p2);
    su->pc=int3d(su->p0,su->p2,0.5);
    p3=int3d(su->p1,su->pc,2);
@@ -552,16 +548,10 @@ int read_nec_SC(char *s)     /* SC -> continuation of SM */
 int read_nec_SM(char *s)     /* SM -> multiple surface patch (shown as one large rectangular piece of surface) */
 {
    Surface *su;
-   if (numsurfaces==MAXSURFACES) return Err_nsurfaces;
+   EXPAND_IF_NECESSARY(numsurfaces,maxsurfaces,surfaces)
    su=surfaces+numsurfaces;
    removecommas(s);
-   if (sscanf(s,"SM%*i%*i%lg%lg%lg%lg%lg%lg",&su->p0.x,&su->p0.y,&su->p0.z,&su->p1.x,&su->p1.y,&su->p1.z) !=6) return Err_scan;
-   su->p0.x*=GeomScale;
-   su->p0.y*=GeomScale;
-   su->p0.z*=GeomScale;
-   su->p1.x*=GeomScale;
-   su->p1.y*=GeomScale;
-   su->p1.z*=GeomScale;
+   if (sscanf(s,"SM%*i%*i%g%g%g%g%g%g",&su->p0.x,&su->p0.y,&su->p0.z,&su->p1.x,&su->p1.y,&su->p1.z) !=6) return Err_scan;
    updateextremes(&su->p0);
    updateextremes(&su->p1);
    return 0;
@@ -574,18 +564,12 @@ int read_nec_SP(char *s)     /* SP -> surface patch */
    double x1,y1,z1,x2,y2,z2;
    double r;
 
-   if (numsurfaces==MAXSURFACES) return Err_nsurfaces;
+   EXPAND_IF_NECESSARY(numsurfaces,maxsurfaces,surfaces)
    su=surfaces+numsurfaces++;
    su->type=SU_arb;
    removecommas(s);
    if (sscanf(s,"SP%*d%d%lg%lg%lg%lg%lg%lg",&ns,&x1,&y1,&z1,&x2,&y2,&z2) !=7) return Err_scan;
    if (ns!=0) { fprintf(stderr,"Only \"arbitrary\" patch shape (NS=0) supported for now.\n"); return Err_misc; }
-   x1*=GeomScale;
-   y1*=GeomScale;
-   z1*=GeomScale;
-   x2*=GeomScale;
-   y2*=GeomScale;
-   z2*=GeomScale;
    su->pc.x=x1; su->pc.y=y1; su->pc.z=z1;
    updateextremes(&su->pc);
    r=sqrt(z2/M_PI);
@@ -608,7 +592,6 @@ void read_nec_input(FILE *f)               /* read the input file, and update th
    int err;
    int last_was_NT_or_TL=0;
 
-   GeomScale=1.0;
    nextabs=1;
    fgets(s,100,f);
    while (!feof(f) && (s[0]!='E' || s[1]!='N')) {
