@@ -1,9 +1,9 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2002, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
  *  This module parses NEC2's output.
  *  Furthermore, it contains some functions for later processing of this
@@ -60,15 +60,136 @@ void *myrealloc(void *p,size_t n)
 }
 
 
+double calc_polfactor(int pol,Gain *g)
+{
+   /* note: for efficiency reasons, these formulas are also in update_maxgains() */
+   double a,b,f;
+   switch (pol) {
+      case POLhor:
+         a=g->axial*g->axial;
+         b=sin(g->tilt);
+         f=(a+(1-a)*b*b)/(1+a);
+         break;
+      case POLvert:
+         a=g->axial*g->axial;
+         b=cos(g->tilt);
+         f=(a+(1-a)*b*b)/(1+a);
+         break;
+      case POLlhcp:
+         a=g->axial*g->axial;
+         f=(1+2*g->axial+a)/2/(1+a);
+         break;
+      case POLrhcp:
+         a=g->axial*g->axial;
+         f=(1-2*g->axial+a)/2/(1+a);
+         break;
+      default: f=1; /* POLnone, POLcolour */
+   }
+   return f;
+}
+
+void update_maxgains(Gain *g,NECoutput *ne,double phi,double theta)
+{
+   double f;
+   double a,b;
+
+   if (g->total > ne->d[neco_maxgain]) {
+      ne->d[neco_maxgain]=g->total;
+      ne->d[neco_phi]=phi;
+      ne->d[neco_theta]=theta;
+   }
+
+   a=g->axial*g->axial;
+
+   f=(1+2*g->axial+a)/2/(1+a);
+   f=g->total+10*log10(f);
+   if (f > ne->d[neco_maxgain_lhcp]) {
+      ne->d[neco_maxgain_lhcp]=f;
+      ne->d[neco_phi_lhcp]=phi;
+      ne->d[neco_theta_lhcp]=theta;
+   }
+
+   f=(1-2*g->axial+a)/2/(1+a);
+   f=g->total+10*log10(f);
+   if (f > ne->d[neco_maxgain_rhcp]) {
+      ne->d[neco_maxgain_rhcp]=f;
+      ne->d[neco_phi_rhcp]=phi;
+      ne->d[neco_theta_rhcp]=theta;
+   }
+
+   b=sin(g->tilt);
+   f=(a+(1-a)*b*b)/(1+a);
+   f=g->total+10*log10(f);
+   if (f > ne->d[neco_maxgain_hor]) {
+      ne->d[neco_maxgain_hor]=f;
+      ne->d[neco_phi_hor]=phi;
+      ne->d[neco_theta_hor]=theta;
+   }
+
+   b=cos(g->tilt);
+   f=(a+(1-a)*b*b)/(1+a);
+   f=g->total+10*log10(f);
+   if (f > ne->d[neco_maxgain_vert]) {
+      ne->d[neco_maxgain_vert]=f;
+      ne->d[neco_phi_vert]=phi;
+      ne->d[neco_theta_vert]=theta;
+   }
+}
+
+void find_fb(NECoutput *ne,Radpattern *r)
+{
+   int i,j,k;
+   double theta,phi;
+
+   theta=ne->d[neco_theta]*M_PI/180;
+   phi=ne->d[neco_phi]*M_PI/180;
+   for (k=POLnone;k<=POLrhcp;k++) {
+
+      for (i=0;i<r->numtheta;i++) {
+         double d;
+         d = fmod(fabs(r->gtheta[i]+theta),2*M_PI);
+         if (fabs(d-M_PI) < 0.00001) break;
+      }
+      for (j=0;j<r->numphi;j++) {
+         double d;
+         d = fmod(fabs(r->gphi[j]-phi),2*M_PI);
+         if (fabs(d-M_PI) < 0.00001) break;
+      }
+
+      if (k==POLnone) {
+         if (i<r->numtheta && j<r->numphi)
+            ne->d[neco_fb] = ne->d[neco_maxgain] - r->gain[j][i].total;
+         else
+            ne->d[neco_fb] = -DBL_MAX;
+      } else {
+         if (i<r->numtheta && j<r->numphi) {
+            double f;
+            Gain *g=&r->gain[j][i];
+            ne->d[Neco_gsize*k+Neco_polfb2] = ne->d[Neco_gsize*k+Neco_polgain] - g->total;
+            f=calc_polfactor(k,g);
+            ne->d[Neco_gsize*k+Neco_polfb1] = ne->d[Neco_gsize*k+Neco_polgain] - r->gain[j][i].total + 10*log10(f);
+         }
+         else {
+            ne->d[Neco_gsize*k+Neco_polfb1] = -DBL_MAX;
+            ne->d[Neco_gsize*k+Neco_polfb2] = -DBL_MAX;
+         }
+      }
+
+      theta=ne->d[Neco_gsize*(k+1)+Neco_poltheta]*M_PI/180;
+      phi=ne->d[Neco_gsize*(k+1)+Neco_polphi]*M_PI/180;
+   }
+}
+
+
 void read_nec_output_rad(FILE *f,NECoutput *ne)           /* tries to read (far field) radiation pattern data from f */
 {
    char s[200];
-   double phi,theta,db;
-   int i,j;
+   double phi,theta;
+   float db,axial,tilt;
+   char polsense;
+   int i;
    int end=0;
    int maxthetas, maxphis;
-   double maxdb_phi,maxdb_theta;
-   double maxdb;
    Radpattern *r;
 
    /* Some versions of NEC2 output extraneous data after "RADIATION PATTERNS" before the actual data
@@ -80,85 +201,74 @@ void read_nec_output_rad(FILE *f,NECoutput *ne)           /* tries to read (far
    /* allocate some memory; several of these arrays may need to be reallocated later on, if it turns out that their initial size is too small */
    r=mymalloc(sizeof(Radpattern));
    maxphis=128;
-   r->gain=mymalloc(maxphis*sizeof(double*));
+   r->gain=mymalloc(maxphis*sizeof(Gain*));
    r->gphi=mymalloc(maxphis*sizeof(double));
 
    /* read the first block of radiation data, i.e., for one value of phi and the full range of theta */
    /* after this, we know how many thetas to expect per phi, which makes the memory allocation simpler */
    fgets(s,200,f);
    removecommas(s);
-   if (sscanf(s,"%lg%lg%*g%*g%lg",&theta,&phi,&db)!=3) return;
-   maxdb=db; maxdb_phi=phi; maxdb_theta=theta;
+   if (sscanf(s,"%lg%lg%*g%*g%g%g%g %c",&theta,&phi,&db,&axial,&tilt,&polsense)!=6) return;
    r->gphi[0]=phi;
    r->numphi=1;
    r->numtheta=0;
    maxthetas=16;
    r->gtheta=mymalloc(maxthetas*sizeof(double));
-   r->gain[0]=mymalloc(maxthetas*sizeof(double));
+   r->gain[0]=mymalloc(maxthetas*sizeof(Gain));
    do {
+      Gain *g;
       if (r->numtheta>=maxthetas) {
          maxthetas*=2;
          r->gtheta=myrealloc(r->gtheta,maxthetas*sizeof(double));
-         r->gain[0]=myrealloc(r->gain[0],maxthetas*sizeof(double));
+         r->gain[0]=myrealloc(r->gain[0],maxthetas*sizeof(Gain));
       }
-      r->gain[r->numphi-1][r->numtheta]=db;
+      g=&r->gain[r->numphi-1][r->numtheta];
+      g->total=db;
+      g->tilt=tilt*M_PI/180;
+      if (polsense=='R') axial=-axial;
+      g->axial=axial;
+      update_maxgains(g,ne,phi,theta);
       r->gtheta[r->numtheta++]=theta*M_PI/180;
-      if (db>maxdb) { maxdb=db; maxdb_phi=phi; maxdb_theta=theta; }
       fgets(s,200,f);
       removecommas(s);
-      if (sscanf(s,"%lg%lg%*g%*g%lg",&theta,&phi,&db)!=3) { end=1; break; }
+      if (sscanf(s,"%lg%lg%*g%*g%g%g%g %c",&theta,&phi,&db,&axial,&tilt,&polsense)!=6) { end=1; break; }
    } while (phi==r->gphi[0]);
    r->gtheta=myrealloc(r->gtheta,r->numtheta*sizeof(double));
-   r->gain[0]=myrealloc(r->gain[0],r->numtheta*sizeof(double));
+   r->gain[0]=myrealloc(r->gain[0],r->numtheta*sizeof(Gain));
    r->gphi[0]*=M_PI/180;
 
    /* next, read the rest of the data, i.e., for the same values of theta and the entire range of phi */
    if (!end) {
       i=0;
       while (1) {
+         Gain *g;
          if (i==0) {
             if (r->numphi>=maxphis) {
                maxphis*=2;
-               r->gain=myrealloc(r->gain,maxphis*sizeof(double*));
+               r->gain=myrealloc(r->gain,maxphis*sizeof(Gain));
                r->gphi=myrealloc(r->gphi,maxphis*sizeof(double));
             }
-            r->gain[r->numphi]=mymalloc(r->numtheta*sizeof(double));
+            r->gain[r->numphi]=mymalloc(r->numtheta*sizeof(Gain));
             r->gphi[r->numphi++]=phi*M_PI/180;
          }
-         r->gain[r->numphi-1][i]=db;
-         if (db>maxdb) { maxdb=db; maxdb_phi=phi; maxdb_theta=theta; }
+         g=&r->gain[r->numphi-1][i];
+         g->total=db;
+         g->tilt=tilt*M_PI/180;
+         if (polsense=='R') axial=-axial;
+         g->axial=axial;
+         update_maxgains(g,ne,phi,theta);
          if (++i==r->numtheta) i=0;
          fgets(s,200,f);
          removecommas(s);
-         if (sscanf(s,"%lg%lg%*g%*g%lg",&theta,&phi,&db)!=3) break;
+         if (sscanf(s,"%lg%lg%*g%*g%g%g%g %c",&theta,&phi,&db,&axial,&tilt,&polsense)!=6) break;
       }
    }
-   r->gain=myrealloc(r->gain,r->numphi*sizeof(double*));
+   r->gain=myrealloc(r->gain,r->numphi*sizeof(Gain));
    r->gphi=myrealloc(r->gphi,r->numphi*sizeof(double));
-   
-   /* find maximum gain etc. */
-   if (maxdb>globalmaxdb) globalmaxdb=maxdb;
-   if (ne->rp==NULL || maxdb>ne->d[neco_maxgain]) {
-      ne->d[neco_maxgain] = maxdb;
-      ne->d[neco_phi] = maxdb_phi;
-      ne->d[neco_theta] = maxdb_theta;
-      maxdb_theta*=M_PI/180;
-      maxdb_phi*=M_PI/180;
-      for (i=0;i<r->numtheta;i++) {
-         double d;
-         d = fmod(fabs(r->gtheta[i]+maxdb_theta),2*M_PI);
-         if (fabs(d-M_PI) < 0.00001) break;
-      }
-      for (j=0;j<r->numphi;j++) {
-         double d;
-         d = fmod(fabs(r->gphi[j]-maxdb_phi),2*M_PI);
-         if (fabs(d-M_PI) < 0.00001) break;
-      }
-      if (i<r->numtheta && j<r->numphi)
-         ne->d[neco_fb] = maxdb-r->gain[j][i];
-      else
-         ne->d[neco_fb] = -DBL_MAX;
-   }
+
+   /* further processing of maximum gain info */
+   if (ne->d[neco_maxgain] > globalmaxdb) globalmaxdb=ne->d[neco_maxgain];
+   find_fb(ne,r);
 
    /* allocate arrays which will later be filled with the grid coordinates in 3D space */
    r->gpo=mymalloc(r->numphi*sizeof(Point*));
@@ -207,6 +317,9 @@ Currents *read_nec_output_seg(FILE *f)           /* tries to read segmentation d
    /* allocate memory */
    cu=mymalloc(sizeof(Currents));
    cu->numseg=0;
+   cu->maxseg=1;
+   cu->s=mymalloc(cu->maxseg*sizeof(*(cu->s)));
+
    cu->maxI=0;
    cu->maxQ=0;
 
@@ -216,10 +329,7 @@ Currents *read_nec_output_seg(FILE *f)           /* tries to read segmentation d
       double dx,dy,dz;
       Segcur *se;
       if (sscanf(s,"%*d%lg%lg%lg%lg%lg%lg",&x,&y,&z,&l,&alpha,&beta)!=6) break;
-      if (cu->numseg>=MAXSEGMENTS) {
-         fprintf(stderr,"Output file contains data for more segments than allowed (%i)\n",MAXSEGMENTS);
-         exit(1);
-      }
+      EXPAND_IF_NECESSARY(cu->numseg,cu->maxseg,cu->s)
       se=cu->s+cu->numseg;
       alpha*=M_PI/180;
       beta*=M_PI/180;
@@ -278,10 +388,7 @@ void read_nec_output_patches(FILE *f,Currents *cu)           /* tries to read su
    do {
       double x,y,z,area;
       if (sscanf(s,"%*d%lg%lg%lg%*g%*g%*g%lg",&x,&y,&z,&area)!=4) break;
-      if (cu->numseg>=MAXSEGMENTS+MAXPATCHES) {
-         fprintf(stderr,"Output file contains data for more segments and surface patches than allowed (%i)\n",MAXSEGMENTS+MAXPATCHES);
-         exit(1);
-      }
+      EXPAND_IF_NECESSARY(cu->numseg,cu->maxseg,cu->s)
       cu->s[cu->numseg].c.x=x;
       cu->s[cu->numseg].c.y=y;
       cu->s[cu->numseg].c.z=z;
@@ -310,17 +417,18 @@ Currents *read_nec_output_currents(FILE *f, Currents *cug)           /* tries to
    } while (!isdigit(*p) && !feof(f));
    if (feof(f)) return NULL;
 
-   /* allocate memory */
+   /* allocate memory and copy the geometry info that we collected earlier: */
    cu=mymalloc(sizeof(Currents));
-
-   /* copy the geometry info that we collected earlier: */
    memcpy(cu,cug,sizeof(Currents));
+   cu->maxseg=cu->numseg;
+   cu->s=mymalloc(cu->maxseg*sizeof(*(cu->s)));
+   memcpy(cu->s,cug->s,cu->maxseg*sizeof(*(cu->s)));
 
    /* read the amplitude and phase for every segment */
    i=0;
    do {
       int j;
-      if (sscanf(s,"%d%*d%*g%*g%*g%*g%*g%*g%lg%lg",&j,&cu->s[i].a,&cu->s[i].phase)!=3) break;
+      if (sscanf(s,"%d%*d%*g%*g%*g%*g%*g%*g%g%g",&j,&cu->s[i].a,&cu->s[i].phase)!=3) break;
       if (i!=j-1) break;
       for (j=0;j<3;j++) {
          cu->s[i].im[j] = cu->s[i].re[j] * cu->s[i].a * sin(cu->s[i].phase*M_PI/180);
@@ -330,7 +438,7 @@ Currents *read_nec_output_currents(FILE *f, Currents *cug)           /* tries to
       i++;
       fgets(s,200,f);
    } while (i<cu->numrealseg && !feof(f));
-   if (i!=cu->numrealseg) { fprintf(stderr,"Segment currents data in output file incorrect or incomplete\n"); free(cu); return NULL; }
+   if (i!=cu->numrealseg) { fprintf(stderr,"Segment currents data in output file incorrect or incomplete\n"); free(cu->s); free(cu); return NULL; }
 
    return cu;
 }
@@ -356,7 +464,7 @@ void read_nec_output_charges(FILE *f, Currents *cu)           /* tries to read s
       int j;
       double a;
       double l;
-      if (sscanf(s,"%d%*d%*g%*g%*g%*g%lg%lg%lg%*g",&j,&cu->s[i].qre,&cu->s[i].qim,&a)!=4) break;
+      if (sscanf(s,"%d%*d%*g%*g%*g%*g%g%g%lg",&j,&cu->s[i].qre,&cu->s[i].qim,&a)!=4) break;
       if (i!=j-1) break;
       /* note: we read charge densities, so we need to multiply them by the segment length to obtain the charges themselves */
       l=0;
@@ -399,6 +507,7 @@ Currents *read_nec_output_patchcurrents(FILE *f, Currents *cu)           /* trie
    /* read the amplitude and phase for every segment */
    i=cu->numrealseg;
    oldnumseg=cu->numseg;
+   EXPAND_IF_NECESSARY(2*cu->numseg-cu->numrealseg,cu->maxseg,cu->s)
    do {
       double xr,xi,yr,yi,zr,zi;  /* real and imaginary components of current density in X, Y and Z direction */
       double xr2,xi2,yr2,yi2,zr2,zi2;  /* and their squares */
@@ -504,7 +613,7 @@ Currents *read_nec_output_patchcurrents(FILE *f, Currents *cu)           /* trie
       fgets(s,200,f);  /* again, skip the line containing only the patch number */
       fgets(s,200,f);
    } while (i<oldnumseg && !feof(f));
-   if (i!=oldnumseg) { fprintf(stderr,"Surface patch currents data in output file incorrect\n"); free(cu); return NULL; }
+   if (i!=oldnumseg) { fprintf(stderr,"Surface patch currents data in output file incorrect\n"); free(cu->s); free(cu); return NULL; }
    
    return cu;
 }
@@ -539,7 +648,7 @@ void read_nec_output_nearfield(FILE *f, NECoutput *ne,int magnetic)      /* trie
       double l;
 
       /* parse a line of data */
-      if (sscanf(s,"%lg%lg%lg%lg%lg%lg%lg%lg%lg",
+      if (sscanf(s,"%g%g%g%lg%lg%lg%lg%lg%lg",
          &p.x, &p.y, &p.z, 
          &mag[0], &phase[0],
          &mag[1], &phase[1],
@@ -616,6 +725,16 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
    int oldnumneco;
 
    oldnumneco=numneco;
+ 
+   /* skip all lines until a line containing "STRUCTURE SPECIFICATION": this
+      effectively skips all comments (from CM cards) and text resulting from
+      reading Numerical Green's Function parts. Of course, if a user writes
+      "STRUCTURE SPECIFICATION" in his comment lines, this still fails...
+   */
+   do {
+      fgets(s,200,f);
+   } while (!feof(f) && !strstr(s,"STRUCTURE SPECIFICATION"));
+   if (feof(f)) return 1;
 
    /* check whether segmentation and surface patch data is included, and if so, read it */
    do {
@@ -625,7 +744,9 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
    } while (!feof(f) && !strstr(s,"FREQUENCY"));
 
    printf("#  freq.       Zr       Zi      SWR     gain      f/b      phi    theta\n");
-   while (numneco<MAXFREQS && !feof(f)) {
+   while (!feof(f)) {
+
+      EXPAND_IF_NECESSARY(numneco,maxfreqs,neco)
    
       /* search for the frequency and read it */
       do fgets(s,200,f);  while (!feof(f) && !strstr(s,"FREQUENCY="));
@@ -642,6 +763,11 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
          ne->cu = NULL;
          ne->nf = NULL;
          ne->maxe = ne->maxh = 0;
+         ne->d[neco_maxgain]=-DBL_MAX;
+         ne->d[neco_maxgain_hor]=-DBL_MAX;
+         ne->d[neco_maxgain_vert]=-DBL_MAX;
+         ne->d[neco_maxgain_lhcp]=-DBL_MAX;
+         ne->d[neco_maxgain_rhcp]=-DBL_MAX;
       } else {
          for (i=0, ne=neco; i<numneco; i++, ne++)
             if (ne->f>=freq) {
@@ -654,6 +780,11 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
                ne->cu = NULL;
                ne->nf = NULL;
                ne->maxe = ne->maxh = 0;
+               ne->d[neco_maxgain]=-9999;
+               ne->d[neco_maxgain_hor]=-9999;
+               ne->d[neco_maxgain_vert]=-9999;
+               ne->d[neco_maxgain_lhcp]=-9999;
+               ne->d[neco_maxgain_rhcp]=-9999;
                break;
             }
       }
@@ -681,6 +812,7 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
          if (cu && strstr(s,"CURRENTS AND LOCATION")) {
             if (ne->cu) {
                fprintf(stderr,"File contains more than one set of currents at a frequency; ignoring earlier data\n");
+               free(ne->cu->s);
                free(ne->cu);
             }
             ne->cu=read_nec_output_currents(f,cu);
@@ -719,7 +851,7 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
          printf("       -        -        -        -\n");
    }
 
-   if (cu) free(cu);
+   if (cu) { free(cu->s); free(cu); }
 
    if (rp_index<0 || rp_index>=numneco) {
       if (new_rp_index>=0) rp_index=new_rp_index;
@@ -732,6 +864,8 @@ int read_nec_output(FILE *f)          /* tries to read NEC output data from f; r
 }
 
 
+#define pow10(x) exp(M_LN10*(x))
+
 void process_nec_output(NECoutput *ne)		/* transform gain distribution into an array of points in 3D space */
 {
    int i,j;
@@ -745,13 +879,22 @@ void process_nec_output(NECoutput *ne)		/* transform gain distribution into an a
    while (rp) {
       for (i=0;i<rp->numtheta;i++)
          for (j=0;j<rp->numphi;j++) {
-            switch (gainscale) {
-               case GSlinpower: r=pow(10.0,(rp->gain[j][i]-globalmaxdb)/10); break;        /* linear in power */
-               case GSlinvolt: r=pow(10.0,(rp->gain[j][i]-globalmaxdb)/20); break;        /* linear in voltage */
-               case GSarrl: r=exp(0.116534*(rp->gain[j][i]-globalmaxdb)/2); break;  /* arrl */
-               case GSlog: r=rp->gain[j][i]-globalmaxdb; if (r<-40) r=0; else r=r/40+1; break;  /* log */
+            double db;
+            double f;
+            Gain *g=&rp->gain[j][i];
+            f=calc_polfactor(polarization,g);
+            if (f<1e-200) r=0;
+            else {
+               db=g->total-globalmaxdb;
+               switch (gainscale) {
+                  case GSlinpower: r=f*pow10(db/10); break;        /* linear in power */
+                  case GSlinvolt: r=sqrt(f)*pow10(db/20); break;        /* linear in voltage */
+                  case GSarrl: r=exp(0.116534*(db+10*log10(f))/2); break;  /* arrl */
+                  case GSlog: r=db+10*log10(f); if (r<-40) r=0; else r=r/40+1; break;  /* log */
+               }
+               r*=extr;
             }
-            r*=extr;
+            if (r<1e-6) r=1e-6;  /* prevent points from being just about exactly at the origin, since this confuses the edge hiding calculations */
             rp->gpo[j][i].x = rp->cos_gphi[j] * rp->sin_gtheta[i] * r;
             rp->gpo[j][i].y = rp->sin_gphi[j] * rp->sin_gtheta[i] * r;
             rp->gpo[j][i].z = rp->cos_gtheta[i] * r;
@@ -777,6 +920,7 @@ void calc_vgain(void)     /* update the vgain records in accordance with the vie
    for (k=0, ne=neco; k<numneco; k++, ne++) {
       rp=ne->rp;
       ne->d[neco_vgain]=-DBL_MAX;
+      ne->d[neco_vgain2]=-DBL_MAX;
       dif=DBL_MAX;
       rp0=NULL;
       while (rp) {
@@ -791,8 +935,11 @@ void calc_vgain(void)     /* update the vgain records in accordance with the vie
             for (j=0;j<rp->numphi;j++) {
                d = d0 + fabs(fmod(ph-rp->gphi[j],2*M_PI)-M_PI);
                if (d<dif) { 
+                  Gain *g;
                   dif=d; j0=j; rp0=rp;
-                  ne->d[neco_vgain]=rp->gain[j0][i0];
+                  g=&rp->gain[j0][i0];
+                  ne->d[neco_vgain]=g->total;
+                  ne->d[neco_vgain2]=g->total+10*log10(calc_polfactor(polarization,g));
                }
             }
          }
@@ -812,10 +959,15 @@ void calc_vgain(void)     /* update the vgain records in accordance with the vie
             d = fmod(fabs(rp0->gphi[j]-ph0),2*M_PI);
             if (fabs(d-M_PI) < 0.00001) break;
          }
-         if (i<rp0->numtheta && j<rp0->numphi)
-            ne->d[neco_vfb] = ne->d[neco_vgain]-rp0->gain[j][i];
-         else
+         if (i<rp0->numtheta && j<rp0->numphi) {
+            Gain *g;
+            g=&rp0->gain[j][i];
+            ne->d[neco_vfb] = ne->d[neco_vgain2]-g->total+10*log10(calc_polfactor(polarization,g));
+            ne->d[neco_vfb2] = ne->d[neco_vgain2]-g->total;
+         } else {
             ne->d[neco_vfb] = -DBL_MAX;
+            ne->d[neco_vfb2] = -DBL_MAX;
+         }
       }
    }
 }
