@@ -1,16 +1,15 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2000, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2003, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
  *  This module contains all low level postscript drawing routines.
  */
 
 #include <stdio.h>
-#include <X11/X.h>
-#include <X11/Xlib.h>
+#include <gdk/gdk.h>
 #include <time.h>
 #include <string.h>
 
@@ -21,6 +20,7 @@ FILE *psfile=NULL;
 
 int needstroke;
 int needmoveto;
+GdkColor *xcprev=NULL;
 
 void ps_DrawLine(double x1,double y1,double x2,double y2)
 {
@@ -43,23 +43,25 @@ void ps_SetLineAttributes(unsigned line_width,int line_style,int cap_style,int j
 					 which are possible on the device! */
    fprintf(psfile,"%u setlinewidth ",line_width);
    switch (line_style) {
-      case LineSolid:
+      case GDK_LINE_SOLID:
          fprintf(psfile,"[] 0 setdash\n");
          break;
-      case LineOnOffDash:
+      case GDK_LINE_ON_OFF_DASH:
          fprintf(psfile,"[5 5] 0 setdash\n");
          break;
    }
-   if (cap_style!=CapButt || join_style!=JoinRound || line_style==LineDoubleDash) {
+   if (cap_style!=GDK_CAP_ROUND || join_style!=GDK_JOIN_ROUND || line_style==GDK_LINE_DOUBLE_DASH) {
       puts("Error in ps_SetLineAttributes!");
    }
    needmoveto=1;
 }
 
-void ps_SetForeground(XColor *xc)
+void ps_SetForeground(GdkColor *xc)
 {
+   if (xc==xcprev) return;
+   xcprev=xc;
    if (needstroke) { fprintf(psfile,"stroke\n"); needstroke=0; }
-   fprintf(psfile,"%g %g %g setrgbcolor\n",xc->red/65536.,xc->green/65536.,xc->blue/65536.);
+   fprintf(psfile,"%g %g %g setrgbcolor\n",xc->red/65535.,xc->green/65535.,xc->blue/65535.);
    needmoveto=1;
 }
 
@@ -79,6 +81,8 @@ void ps_Complete(void)
 
 void ps_SetClipRectangle(double x1,double y1,double x2,double y2)
 {
+   if (needstroke) { fprintf(psfile,"stroke\n"); needstroke=0; }
+   xcprev=NULL;
    fprintf(psfile,"grestore gsave\n");
    fprintf(psfile,"newpath %g %g moveto %g %g lineto %g %g lineto %g %g lineto closepath clip newpath\n",
                            x1,y1,       x1,y2,       x2,y2,       x2,y1 );
@@ -136,13 +140,14 @@ int write_postscript(char *filename,void (*drawfn)(int),int xsize,int ysize)
       (double)PSlly+ysize );
    fprintf(psfile,"newpath 0 0 moveto 0 %g lineto %g %g lineto %g 0 lineto closepath clip newpath\n",
                   (double)ysize, (double)xsize, (double)ysize, (double)xsize );
-   fprintf(psfile,"1 setlinejoin 0 setlinecap\n");
+   fprintf(psfile,"1 setlinejoin 1 setlinecap\n");
    fprintf(psfile,"/%s findfont %i scalefont setfont\n",PSFONT,fontheight);    /* prepare the font */
    fprintf(psfile,"gsave 0 0 moveto (10ALIZgjy) true charpath pathbbox grestore /ascent exch def pop -1 mul /descent exch def pop\n");  /* estimate ascent and descent of font */
    fprintf(psfile,"gsave\n");  /* this gsave will be grestore'd by the first SetClipRectangle call */
    
    needstroke=0;
    needmoveto=0;
+   xcprev=NULL;
 
    drawfn(0);
 
