@@ -1,11 +1,12 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2003, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
- *  Main module.
+ *  Main module, which mostly deals with initial setup, (re)reading of
+ *  files, and parsing command line options.
  */
 
 
@@ -30,7 +31,7 @@ int structplot;
 int scaleplot=0;
 int phasedirlock=0;
 double phasephi, phasetheta;
-int polarization=POLhor;
+int polarization=POLnone;
 int distcor=1;
 double phaseoffset=0;
 int maxthickness=10;
@@ -58,24 +59,30 @@ char *exppng=NULL;
 /*---------- global variables, together containing the antenna structure -------------*/
 
 int numwires=0;
-Wire wires[MAXWIRES];
+int maxwires=1;
+Wire *wires=NULL;
 
 int numsurfaces=0;
-Surface surfaces[MAXSURFACES];
+int maxsurfaces=1;
+Surface *surfaces;
 
 int numexcis=0;
-Exci excis[MAXEXCIS];
+int maxexcis=1;
+Exci *excis=NULL;
 
 int numloads=0;
-Load loads[MAXLOADS];
+int maxloads=1;
+Load *loads=NULL;
 
 int numnetws=0;
-Netw netws[MAXNETWS];
+int maxnetws=1;
+Netw *netws=NULL;
 
 /*---------- global variables, together containing the gain distribution -------------*/
 
-NECoutput neco[MAXFREQS];
 int numneco=0;
+int maxfreqs=1;
+NECoutput *neco=NULL;
 
 int rp_index=-1;       /* index of the entry in neco[] whose radiation pattern is being shown in the 3D plot */
 
@@ -84,7 +91,7 @@ int rp_index=-1;       /* index of the entry in neco[] whose radiation pattern i
 
 void info()
 {
-   puts("Copyright (C) 1998-2001 Pieter-Tjerk de Boer -- pa3fwm@amsat.org\n");
+   puts("Copyright (C) 1998-2003 Pieter-Tjerk de Boer -- pa3fwm@amsat.org\n");
    puts("Xnecview comes with ABSOLUTELY NO WARRANTY. This is free software, and you are\n"
         "welcome to redistribute it under the conditions of version 2 of the GNU GPL.\n"
         "For more information: see the files README and COPYING which should accompany\n"
@@ -104,6 +111,7 @@ void usage()
         "  --animation  : set structure view to 'animation'\n"
         "  --slice      : set radiation view to 'slice'\n"
         "  --frame      : set radiation view to 'frame'\n"
+        "  --opaque     : set radiation view to 'opaque'\n"
         "  --near       : set radiation view to 'near field'\n"
         "  --linpower   : set radiation scale linear in power\n"
         "  --linvoltage : set radiation scale linear in voltage\n"
@@ -181,7 +189,10 @@ void reread(void)
          rp=rp->next;
          free(rp1);
       }
-      if (neco[numneco].cu) free(neco[numneco].cu);
+      if (neco[numneco].cu) {
+         free(neco[numneco].cu->s);
+         free(neco[numneco].cu);
+      }
       nf=neco[numneco].nf;
       while (nf) {
          nf1=nf;
@@ -216,8 +227,9 @@ void process_options(void)
 #ifndef NO_GETOPT
    struct option longopts[]={
       { "near",     0, &gainplot, GPnearfield },
-      { "slice",    0, &gainplot, GP2d },
-      { "frame",    0, &gainplot, GP3d },
+      { "slice",    0, &gainplot, GPslice },
+      { "frame",    0, &gainplot, GPframe },
+      { "opaque",   0, &gainplot, GPopaque },
 
       { "struct",    0, &structplot, SPstruct },
       { "tags",      0, &structplot, SPtags },
@@ -229,6 +241,8 @@ void process_options(void)
       { "arrl", 0, &gainscale, GSarrl },
       { "log",  0, &gainscale, GSlog },
 
+      { "pol", 1, NULL, 40 },
+
       { "iscale", 1, NULL, 13 },
       { "qscale", 1, NULL, 14 },
       { "escale", 1, NULL, 15 },
@@ -278,6 +292,17 @@ void process_options(void)
          case 31: exppng=optarg; break;
 #endif
          case 32: r0=atof(optarg); break;
+         case 40: if (!strcmp(optarg,"total")) polarization=POLnone;
+                  else if (!strcmp(optarg,"hor")) polarization=POLhor;
+                  else if (!strcmp(optarg,"vert")) polarization=POLvert;
+                  else if (!strcmp(optarg,"lhcp")) polarization=POLlhcp;
+                  else if (!strcmp(optarg,"left")) polarization=POLlhcp;
+                  else if (!strcmp(optarg,"rhcp")) polarization=POLrhcp;
+                  else if (!strcmp(optarg,"right")) polarization=POLrhcp;
+                  else if (!strcmp(optarg,"colour")) polarization=POLcolour;
+                  else if (!strcmp(optarg,"color")) polarization=POLcolour;
+                  break;
+
          case 'h': info(); usage(); break;
          case '?': printf("Try `%s -h' for help.\n",myargv[0]); break;
       }
@@ -314,10 +339,10 @@ void process_optionstring(char *s)
    s=ar;
    av=mymalloc((1+strlen(s))*sizeof(char*));
    ac=0;
-   s=strtok(s," \n");
+   s=strtok(s," \n\r");
    while (s) {
       av[ac++]=s;
-      s=strtok(NULL," ");
+      s=strtok(NULL," \n\r");
    }
    if (ac+myargc<=Maxopts) {
       memmove(myargv+optind+ac, myargv+optind, (myargc-optind)*sizeof(char*));
@@ -342,6 +367,13 @@ int main(int argc,char **argv)
    initX(&argc,argv);
    inputfilename=NULL;
 
+   wires=mymalloc(maxwires*sizeof(Wire));
+   surfaces=mymalloc(maxsurfaces*sizeof(Surface));
+   excis=mymalloc(maxexcis*sizeof(Exci));
+   loads=mymalloc(maxloads*sizeof(Load));
+   netws=mymalloc(maxnetws*sizeof(Netw));
+   neco=mymalloc(maxfreqs*sizeof(NECoutput));
+
    structplot=-1;
    gainplot=-1;
    g_argc=0;
@@ -363,7 +395,7 @@ int main(int argc,char **argv)
          else {
             for (i=0;i<numneco;i++)
                if (neco[i].rp || neco[i].nf) {
-                  gainplot=GP3d;
+                  gainplot=1111;
                   break;
                }
             if (gainplot<0) gainplot=GPnone;
@@ -382,8 +414,10 @@ int main(int argc,char **argv)
    } else {
       for (i=0;i<numneco;i++)
          if (neco[i].cu) {
-            window1open=1;
-            if (structplot<0) structplot=SPcurrents;
+            if (structplot<0 && gainplot<0) {
+               structplot=SPcurrents;
+               window1open=1;
+            }
             break;
          }
       if (rp_index>=0) window1open=1;
@@ -394,11 +428,29 @@ int main(int argc,char **argv)
       window2open=1;
       if (neco[rp_index].rp==NULL) { plot2_maxgain=0; plot2_z=1; }
    }
+
    if (numneco==1 && !window1open) {
       fprintf(stderr,"No data available which can be displayed graphically.\n");
       return 0;
    }
 
+   if (gainplot==GPopaque || gainplot==1111) {
+      char *s;
+      Radpattern *rp;
+      rp=neco[rp_index].rp;
+      s=opaque_impossible(rp);
+      if (s) {
+         fprintf(stderr,"Opaque gain plot not possible; reason: %s.\n",s);
+         gainplot=GPframe;
+      } else if (gainplot==1111) {
+         if (rp->next) {
+            gainplot=GPframe;
+            fprintf(stderr,"Opaque gain plot off by default, because of multiple RP cards.\n");
+         } else {
+            gainplot=GPopaque;
+         }
+      }
+   }
 
    calcproj();
    calc_vgain();
