@@ -1,9 +1,9 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2003, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  */
 
 /* ------------------------------------------------------------------------ */
@@ -12,11 +12,11 @@
 #include "config.h"        /* file defining things like default colours etc. */
 
 
-#define VERSION "1.31"
+#define VERSION "1.34"
 
 
 typedef struct {     /* structure to describe a point in 3D space */ 
-   double x,y,z;
+   float x,y,z;
 } Point;
 
 typedef struct {     /* structure to describe a straight wire (segment) */
@@ -79,15 +79,15 @@ extern char *GSnames[];      /* array with names of gainscales */
 extern char GSnumbers[][32]; /* array with descriptions of levels of scale lines in plot */
 
 extern int gainplot;         /* type of radiation plot: */
-enum { GPnone,GP2d,GP3d,GPnearfield,numGP };
+enum { GPnone,GPslice,GPframe,GPopaque,GPnearfield,numGP };
 
 extern int structplot;       /* type of structure plot: */
 enum { SPnone,SPstruct,SPtags,SPcurrents,SPanim,numSP };
 
 extern int scaleplot;        /* flag: plot gain scale? */
 
-extern int polarization;     /* polarization for currents display */
-enum { POLnone,POLhor,POLvert,POLlhcp,POLrhcp };
+extern int polarization;     /* polarization for gain and currents displays */
+enum { POLnone,POLhor,POLvert,POLlhcp,POLrhcp,POLcolour };   /* note: if this order changes, find_fb() may also need to be changed */
 
 extern int phasedirlock;     /* flag: is the direction used for calculation phases locked? */
 extern double phasephi,phasetheta;   /* if so, this is that direction; otherwise, it is identical to phi and theta */
@@ -111,22 +111,32 @@ extern int win2sizex,win2sizey; /* size of freqplot window in pixels */
 extern int plot2_swr, plot2_z, plot2_z2, plot2_maxgain, plot2_dir, plot2_vgain; /* flags for the each of the frequency plots in window 2 */
 
 extern int numwires;
-extern Wire wires[MAXWIRES];
+extern int maxwires;
+extern Wire *wires;
 extern int numsurfaces;
-extern Surface surfaces[MAXSURFACES];
+extern int maxsurfaces;
+extern Surface *surfaces;
 extern int numexcis;
-extern Exci excis[MAXEXCIS];
+extern int maxexcis;
+extern Exci *excis;
 extern int numloads;
-extern Load loads[MAXLOADS];
+extern int maxloads;
+extern Load *loads;
 extern int numnetws;
-extern Netw netws[MAXNETWS];
+extern int maxnetws;
+extern Netw *netws;
 
 
+typedef struct {     /* structure to contain, for one direction, the radiated power and polarization information */
+   float total;   /* total radiated power, dBi */
+   float axial;   /* axial ratio, with sign indicating the polarity: + left-hand circular, - right-hand */
+   float tilt;    /* tilt angle of major axis, radians; defined as in NEC output: 0 = vertical, >0 topleft/bottomright */
+} Gain;
 
 typedef struct RAdpattern {    /* structure to contain (far field) radiation pattern for one frequency */
    double *gtheta;  /* pointer to array of theta values */
    double *gphi;    /* pointer to array of phi values */
-   double **gain;   /* pointer to an array of pointers (one for every phi) to an array of gain values (one for every theta at that phi) */
+   Gain **gain;     /* pointer to an array of pointers (one for every phi) to an array of Gain structs (one for every theta at that phi) */
    Point **gpo;     /* pointer to an array of pointers (one for every phi) to an array of Points (one for every theta at that phi) */
    int numtheta;    /* number of theta values */
    int numphi;      /* number of phi values */
@@ -162,17 +172,18 @@ typedef struct {   /* structure to contain the current (phase, magnitude, locati
    */
    Point p0,p1;  /* coordinates of segment endpoints */
    Point c;      /* coordinate of segment center point */
-   double a;     /* amplitude of the current */
-   double phase; /* phase of the current (degrees) */
-   double dist;  /* distance of center of segment from the plane through the origin, perpendicular to the viewing direction; positive if in front of this plane, negative otherwise */
-   double area;  /* area of a surface patch */
-   double re[3],im[3]; /* real and imaginary vector */
-   double q0[3],q1[3]; /* two unity vectors, orthogonal to each other and to the direction of the segment; used for drawing charges */
-   double qre,qim;  /* real and imaginary value of the charge */
+   float a;     /* amplitude of the current */
+   float phase; /* phase of the current (degrees) */
+   float dist;  /* distance of center of segment from the plane through the origin, perpendicular to the viewing direction; positive if in front of this plane, negative otherwise */
+   float area;  /* area of a surface patch */
+   float re[3],im[3]; /* real and imaginary vector */
+   float q0[3],q1[3]; /* two unity vectors, orthogonal to each other and to the direction of the segment; used for drawing charges */
+   float qre,qim;  /* real and imaginary value of the charge */
 } Segcur;
 
 typedef struct {   /* structure to contain the current distribution for all segments at one frequency */
-   Segcur s[MAXSEGMENTS+2*MAXPATCHES];
+   Segcur *s;
+   int maxseg;       /* current size of s[] */
    int numseg;       /* number of valid entries in s[] */
    int numanimseg;   /* number of those that are needed for the animations; i.e., excluding the extra segments caused by the decomposition of the surface patch currents */
    int numrealseg;   /* number of those that are actually segments; so s[numrealseg...numseg-1] are surface patches */
@@ -184,11 +195,11 @@ typedef struct {   /* structure to contain the current distribution for all segm
 typedef struct NF { /* structure to contain the electric and magnetic field at some point (near the antenna) */
    Point p;           /* location of the point */
    int evalid,hvalid; /* flags to indicate presence of E and H data */
-   double ere[3];     /* real component of E vector */
-   double eim[3];     /* imaginary component of E vector */
-   double hre[3];     /* real component of H vector */
-   double him[3];     /* imaginary component of H vector */
-   double poy[3];     /* time-averaged Poynting vector */
+   float ere[3];     /* real component of E vector */
+   float eim[3];     /* imaginary component of E vector */
+   float hre[3];     /* real component of H vector */
+   float him[3];     /* imaginary component of H vector */
+   float poy[3];     /* time-averaged Poynting vector */
    struct NF *next;   /* pointer to next datapoint */
 } Nearfield;
 
@@ -202,25 +213,40 @@ enum {
    neco_maxgain, neco_fb,  /* index of maximum gain and corresponding front/back ratio */
    neco_phi, neco_theta,   /* index direction of maximum gain */
    neco_vgain, neco_vfb,   /* index of gain in viewing direction and corresponding f/b ratio */
+   neco_vgain2, neco_vfb2, /* same, but taking polarization into account; fb/fb2 as fb1/fb2 below */
+   neco_maxgain_hor, neco_fb_hor1, neco_fb_hor2, neco_phi_hor, neco_theta_hor,
+   neco_maxgain_vert, neco_fb_vert1, neco_fb_vert2, neco_phi_vert, neco_theta_vert,
+   neco_maxgain_lhcp, neco_fb_lhcp1, neco_fb_lhcp2, neco_phi_lhcp, neco_theta_lhcp,
+   neco_maxgain_rhcp, neco_fb_rhcp1, neco_fb_rhcp2, neco_phi_rhcp, neco_theta_rhcp,
    neco_numdat             /* last element of the enum: number of elements in the array */
 };
 /* since (maximum) gain and corresponding quantities are not available in all NECoutput records,
    the following macro is needed to tell whether a quantity is available always, or only if
    radiation pattern data is available in the record: */
 #define ONLY_IF_RP(i) ((i)>=neco_maxgain)
+/* a constant for more systematic access to the polarization-specific fields: */
+#define Neco_gsize 5     /* group size */
+#define Neco_polgain (neco_maxgain_hor-Neco_gsize)
+#define Neco_polfb1 (neco_fb_hor1-Neco_gsize)
+#define Neco_polfb2 (neco_fb_hor2-Neco_gsize)
+#define Neco_polphi (neco_phi_hor-Neco_gsize)
+#define Neco_poltheta (neco_theta_hor-Neco_gsize)
+/* now, we can write say  neco_phi_lhcp  as  Neco_gsize*POLlhcp+Neco_polphi  */
+/* note: fb...1 is f/b ratio for this polarization; fb...2 is same but for back total power is used */
 
 typedef struct {    /* structure to contain all output data that NEC produces for one frequency */
    double f;          /* frequency in MHz */
    Radpattern *rp;    /* pointer to radiation pattern data, if available; NULL otherwise */
    double d[neco_numdat];
    int rpgpovalid;    /* flag: !=0 if the gpo field of *rp already contains valid data for the present settings */
-   Currents *cu;      /* pointer to current data, if available; NULL otherwise */
-   Nearfield *nf;     /* pointer to nearfield data, if avaialble; NULL otherwise */
+   Currents *cu;      /* pointer to currents data, if available; NULL otherwise */
+   Nearfield *nf;     /* pointer to nearfield data, if available; NULL otherwise */
    double maxe,maxh;  /* largest values of E and H in nearfield data */
 } NECoutput;
 
-extern NECoutput neco[MAXFREQS];
+extern NECoutput *neco;
 extern int numneco;
+extern int maxfreqs;
 
 extern double globalmaxdb; /* maximum gain, global over all frequencies and all output data files */
 
@@ -236,28 +262,9 @@ extern char *inputfilename;
 
 extern int fontheight;
 
-#ifndef XColor
-   #include <X11/Xlib.h>
+#ifndef GdkColor
+   #include <gdk/gdk.h>
 #endif
-/*
-extern XColor c_bg;
-extern XColor c_axis;
-extern XColor c_wire;
-extern XColor c_surf;
-extern XColor c_back;
-extern XColor c_gain;
-extern XColor c_scale;
-extern XColor c_exci;
-extern XColor c_load;
-extern XColor c_netw;
-extern XColor c_currents[NC_phase];
-extern XColor c_inactive;
-extern XColor c_efield;
-extern XColor c_hfield;
-extern XColor c_poynting;
-extern XColor c_qpos;
-extern XColor c_qneg;
-*/
 #define c_bg    col[0]
 #define c_axis  col[1]
 #define c_wire  col[2]
@@ -274,16 +281,19 @@ extern XColor c_qneg;
 #define c_poynting col[13]
 #define c_qpos     col[14]
 #define c_qneg     col[15]
-#define c_currents (col+16)
-#define NC NC_phase+16
-extern XColor col[NC];
+#define c_gain_lin   col[16]
+#define c_gain_lhcp  col[17]
+#define c_gain_rhcp  col[18]
+#define c_currents (col+19)
+#define NC NC_phase+19
+extern GdkColor col[NC];
 
 
 
 typedef struct {
    void (*SetLineAttributes)(unsigned int,int,int,int);
    void (*DrawLine)(double,double,double,double);
-   void (*SetForeground)(XColor *);
+   void (*SetForeground)(GdkColor *);
    void (*ClearWindow)();
    void (*DrawString)(double,double,char*,double,double);
    void (*Complete)();
@@ -348,4 +358,25 @@ double xfreq(int x);                        /* return frequency corresponding to
 int freqx(double f);                        /* return X coordinate corresponding to frequency */
 int freqindex(double f);                    /* return index in neco[] of data closest to frequency f */
 
-void *mymalloc(size_t n);                   /* as malloc(), but exits with error message if memory not available; implementation in parse_output.c */
+void *mymalloc(size_t n);                   /* like malloc(), but exits with error message if memory not available; implementation in parse_output.c */
+void *myrealloc(void *,size_t n);           /* like realloc(), but exits with error message if memory not available; implementation in parse_output.c */
+
+void draw_opaque(Radpattern *);             /* draw radiation pattern as an opaque surface */
+
+void setpolcolour(double axial);            /* set the colour corresponding to this axial ratio */
+
+char *opaque_impossible(Radpattern *rp);    /* returns NULL if opaque drawing of this Radpattern is possible; otherwise, pointer to string with explanation */
+
+
+/* a macro for conveniently expanding array sizes: */
+#define EXPAND_IF_NECESSARY(num,max,pointer)                \
+   if (num>=max) {                                          \
+      while (max<num+1) max=max*2;                          \
+      pointer=myrealloc(pointer,max*(sizeof(*pointer)));    \
+   }
+
+/* if your system doesn't have the function drem() available, uncomment
+   the following line and recompile (after deleting *.o)
+*/
+/*  #define drem(x,y) ( (x) - (y)*rint((x)/(y)) )   */
+
