@@ -1,22 +1,15 @@
 /*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-2001, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2003, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
- *  Distributed on the conditions of the GPL: see the files README and
- *  COPYING, which accompany this source file.
+ *  Distributed on the conditions of version 2 of the GPL: see the files
+ *  README and COPYING, which accompany this source file.
  *
  *  This module contains all X-windows related stuff; this includes all
  *  event handlers, and thus most of the interaction with the user.
  */
 
 
-#include  <X11/StringDefs.h> 
-#include  <X11/Intrinsic.h> 
-#include  <X11/Core.h> 
-#include  <X11/Xaw/Box.h> 
-#include  <X11/Xlib.h> 
-#include  <X11/keysym.h> 
-#include  <X11/Shell.h> 
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,23 +27,20 @@
 
 
 
-Display *display;
-Drawable window,window2;    /* note: window2, drawing2, gc2, etc. refer to the second window, i.e., the window containing the plots of SWR etc. vs. frequency */
-Widget drawing,drawing2;    /* these are Xlib widgets for the actual graphical drawing, i.e., the windows excluding the buttons */
-GtkWidget *gdraw1=NULL,*gdraw2=NULL;  /* and these are the corresponding GTK widgets */
+/* note: gdraw2, ggc2, etc. refer to the second window, i.e., the window containing the plots of SWR etc. vs. frequency */
+GtkWidget *gdraw1=NULL,*gdraw2=NULL;
 GtkWidget *vbox1;    /* the (GTK) vertical box for window 1, containing the top row of buttons, the drawing itself, and the bottom row of buttons */
 GtkWidget *botrow_curr;   /* the (GTK) bottom row of buttons in window 1 for settings of the currents display */
 GtkWidget *botrow_anim;   /* the (GTK) bottom row of buttons in window 1 for settings of the (animated) near field display */
-int scrnr;
-GC gc,gc2;
+GdkGC *ggc,*ggc2;
+GdkFont *gfont;
 
-Pixmap backg=0;          /* picture buffer used for double buffering code; it will serve as the background pixmap for the window */
-GdkPixmap *gbackg;       /* same, but in GDK's terms */
+GdkPixmap *gbackg=NULL;       /* picture buffer used for double buffering code; it will serve as the background pixmap for the window */
 GtkLabel *msgwidget;     /* label widget in top right corner, used for several messages */
-XFontStruct *fontinfo;   /* info about the font used */
 int fontheight;
-Pixmap backg2=0;         /* for window2, we draw into this pixmap */
-GdkPixmap *gbackg2;      /* same, but in GDK's terms */
+GdkPixmap *gbackg2=NULL;      /* for window2, we draw into this pixmap */
+GdkColormap *gcm;
+int depth;
 
 int redraw=1;        /* flag which signifies need for redrawing of struct/gain plot */
 int dragging=0;      /* flag to indicate that user is dragging the struct/gain plot */
@@ -59,42 +49,86 @@ int quit=0;          /* flag to indicate that the user wants to quit */
 int redraw2=1;       /* flag which signifies need for redrawing of frequency plots; extra for speedup: if it ==2, then only the vgain plot needs to be redrawn */
 int animbusy=0;      /* flag to indicate that we're still busy processing the previous animation picture, to slow down the animation sufficiently on slow machines and/or complicated models */
 
+double old_animfreq=0;   /* storage for animation frequency when animation is temporarily suspended using the 'z' key */
+
 Outdev *out=NULL;
 
 
 /* color values for several items: */
-XColor col[NC];
+GdkColor col[NC];
+GdkColor gcol[NC];
 
 
 /* -------------------- X windows stuff: drawing ---------------------------------------- */
 
 void X_SetLineAttributes(unsigned int a,int b,int c,int d)
 {
-   XSetLineAttributes(display, gc, a,b,c,d);
+   gdk_gc_set_line_attributes(ggc, a,b,c,d);
 }
 
 void X_DrawLine(double a,double b,double c,double d)
 {
-   XDrawLine(display,backg,gc,(int)(a+0.5),(int)(b+0.5),(int)(c+0.5),(int)(d+0.5));
+#if 0
+   if (a>32000 || a<-32000 
+      || b>32000 || b<-32000 
+      || c>32000 || c<-32000 
+      || d>32000 || d<-32000 
+      )
+#endif
+   if (a>winsizex || a<0 
+      || b>winsizey || b<0 
+      || c>winsizex || c<0 
+      || d>winsizey || d<0 
+      ) 
+      {
+      /* clipping is needed, otherwise we run into problems when feeding these extremely large coordinates to the X server; apparently, the X11 protocol uses only 16 bits for the coordinates */
+      double h;
+
+      if (a>winsizex && c>winsizex) return;
+      if (a<0 && c<0) return;
+      if (b>winsizey && d>winsizey) return;
+      if (b<0 && d<0) return;
+
+      if (c<a) { h=a; a=c; c=h; h=b; b=d; d=h; }
+      if (a<0) {
+         b -= (d-b)*a/(c-a);
+         a=0;
+      }
+      if (c>winsizex) {
+         d -= (d-b)*(c-winsizex)/(c-a);
+         c=winsizex;
+      }
+
+      if (d<b) { h=a; a=c; c=h; h=b; b=d; d=h; }
+      if (b<0) {
+         a -= (c-a)*b/(d-b);
+         b=0;
+      }
+      if (d>winsizey) {
+         c -= (c-a)*(d-winsizey)/(d-b);
+         d=winsizey;
+      }
+   }
+   gdk_draw_line(gbackg,ggc,(int)(a+0.5),(int)(b+0.5),(int)(c+0.5),(int)(d+0.5));
 }
 
-void X_SetForeground(XColor *xc)
+void X_SetForeground(GdkColor *xc)
 {
-   XSetForeground(display,gc,xc->pixel);
+   gdk_gc_set_foreground(ggc,xc);
 }
 
 void X_ClearWindow()
 {
    X_SetForeground(&c_bg);
-   XFillRectangle(display,backg,gc, 0,0, winsizex, winsizey);
+   gdk_draw_rectangle(gbackg,ggc,TRUE,0,0,winsizex,winsizey);
 }
 
 
 void X_DrawString(double a,double b,char *s,double d,double e)    /* draw string */
 {
-   if (d>0) a-=d*XTextWidth(fontinfo,s,strlen(s));
-   b += -fontinfo->descent + e*(fontinfo->descent+fontinfo->ascent);
-   XDrawString(display,backg,gc,(int)(a+0.5),(int)(b+0.5),s,strlen(s));
+   if (d>0) a-=d*gdk_string_width(gfont,s);
+   b += -gfont->descent + e*(gfont->descent+gfont->ascent);
+   gdk_draw_string(gbackg,gfont,ggc,(int)(a+0.5),(int)(b+0.5),s);
 }
 
 
@@ -109,50 +143,50 @@ Outdev outX={ X_SetLineAttributes, X_DrawLine, X_SetForeground, X_ClearWindow, X
 
 void X2_SetLineAttributes(unsigned int a,int b,int c,int d)
 {
-   XSetLineAttributes(display, gc2, a,b,c,d);
+   gdk_gc_set_line_attributes(ggc2, a,b,c,d);
 }
 
 void X2_DrawLine(double a,double b,double c,double d)
 {
    if (b<-32000 || d<-32000 || b>32000 || d>32000) return;
-   XDrawLine(display,backg2,gc2,(int)(a+0.5),(int)(b+0.5),(int)(c+0.5),(int)(d+0.5));
+   gdk_draw_line(gbackg2,ggc2,(int)(a+0.5),(int)(b+0.5),(int)(c+0.5),(int)(d+0.5));
 }
 
-void X2_SetForeground(XColor *xc)
+void X2_SetForeground(GdkColor *xc)
 {
-   XSetForeground(display,gc2,xc->pixel);
+   gdk_gc_set_foreground(ggc2,xc);
 }
 
 void X2_SetClipRectangle(double x1,double y1,double x2,double y2)
 {
-   XRectangle xr;
-   xr.x=(int)(x1+0.5);
-   xr.y=(int)(y1+0.5);
-   xr.width=(int)(x2-xr.x+1.5);
-   xr.height=(int)(y2-xr.y+1.5);
-   XSetClipRectangles(display,gc2,0,0,&xr,1,Unsorted);
+   GdkRectangle gr;
+   gr.x=(int)(x1+0.5);
+   gr.y=(int)(y1+0.5);
+   gr.width=(int)(x2-gr.x+1.5);
+   gr.height=(int)(y2-gr.y+1.5);
+   gdk_gc_set_clip_rectangle(ggc2,&gr);
 }
 
 void X2_ClearWindow()
 {
    X2_SetClipRectangle(0,0,win2sizex,win2sizey);
    X2_SetForeground(&c_bg);
-   XFillRectangle(display,backg2,gc2, 0,0, win2sizex, win2sizey);
+   gdk_draw_rectangle(gbackg2,ggc2,TRUE,0,0,win2sizex,win2sizey);
 }
 
 void X2_ClearRectangle(double x1,double y1,double x2,double y2)
 {
    X2_SetClipRectangle(0,0,win2sizex,win2sizey);
    X2_SetForeground(&c_bg);
-   XFillRectangle(display,backg2,gc2, (int)(x1+0.5),(int)(y1+0.5), (int)(x2-x1+1.5),(int)(y2-y1+1.5));
+   gdk_draw_rectangle(gbackg2,ggc2,TRUE, (int)(x1+0.5),(int)(y1+0.5), (int)(x2-x1+1.5),(int)(y2-y1+1.5));
 }
 
 
 void X2_DrawString(double a,double b,char *s,double d,double e)    /* draw string */
 {
-   if (d>0) a-=d*XTextWidth(fontinfo,s,strlen(s));
-   b += -fontinfo->descent + e*(fontinfo->descent+fontinfo->ascent);
-   XDrawString(display,backg2,gc2,(int)(a+0.5),(int)(b+0.5),s,strlen(s));
+   if (d>0) a-=d*gdk_string_width(gfont,s);
+   b += -gfont->descent + e*(gfont->descent+gfont->ascent);
+   gdk_draw_string(gbackg2,gfont,ggc2,(int)(a+0.5),(int)(b+0.5),s);
 }
 
 void X2_Complete(void)
@@ -227,8 +261,7 @@ int write_png(int which,char *filename)
    f=fopen(filename,"w");
    if (!f) return 1;
 
-   X_Complete(); /* hack, needed until we convert the background pixmaps to GTK */
-   image = gdk_image_get(w, 0, 0, width, height);
+   image = gdk_image_get(gbackg, 0, 0, width, height);
    if (!image) {
       fclose(f);
       return 1;
@@ -296,7 +329,7 @@ void upd_msg(void)
 {
    char s[32];
 return;
-   if (scaleplot && (gainplot==GP2d || gainplot==GP3d)) {
+   if (scaleplot && (gainplot==GPslice || gainplot==GPframe || gainplot==GPopaque)) {
       gtk_label_set_text(msgwidget,GSnumbers[gainscale]);
    } else {
       sprintf(s,"phi=%g  theta=%g",rint(phi),rint(theta)); 
@@ -310,10 +343,12 @@ void setrpfreq(void)
    int x;
    redraw=1;
    process_nec_output(neco+rp_index);
-   XClearWindow(display,window2);
-   XSetForeground(display,gc2,c_back.pixel);
-   x=freqx(neco[rp_index].f);
-   XDrawLine(display,window2,gc2,x,0,x,win2sizey);
+   if (window2open) {
+      gdk_window_clear(gdraw2->window);
+      gdk_gc_set_foreground(ggc2,&c_back);
+      x=freqx(neco[rp_index].f);
+      gdk_draw_line(gdraw2->window,ggc2,x,0,x,win2sizey);
+   }
 }
 
 
@@ -321,17 +356,18 @@ void setrpfreq(void)
 
 gint resize_event(GtkWidget *w,GdkEventConfigure *ev,gpointer dummy)
 {
-   if (!backg) return TRUE;
+   if (!gbackg) return TRUE;
    if (winsizex==ev->width && winsizey==ev->height) return TRUE;
    winsizex=ev->width;
    winsizey=ev->height;
    gdk_pixmap_unref(gbackg);
-   gbackg=gdk_pixmap_new(w->window,winsizex,winsizey,DefaultDepth(display,scrnr));
-   backg=GDK_WINDOW_XWINDOW(gbackg);
+   gbackg=gdk_pixmap_new(w->window,winsizex,winsizey,depth);
    calcproj();
    redraw=1;
+#if 0
    w->requisition.width=winsizex;
    w->requisition.height=winsizey;
+#endif
    return TRUE;
 }
 
@@ -417,11 +453,11 @@ gint motion_event(GtkWidget *w,GdkEventMotion *ev,gpointer dummy)
          phase=((i*360.0/NC_phase)-phaseoffset)*M_PI/180.0;
          x=xc+r*cos(phase);
          y=yc+r*sin(phase);
-         XClearWindow(display,window);
-         XSetForeground(display,gc,BlackPixel(display,scrnr));
-         XDrawLine(display,window,gc,xc,yc,x,y);
+         gdk_window_clear(gdraw1->window);
+         gdk_gc_set_foreground(ggc,&c_axis);   /* black */
+         gdk_draw_line(gdraw1->window,ggc,xc,yc,x,y);
       } else if (i==-1) {
-         XClearWindow(display,window);
+         gdk_window_clear(gdraw1->window);
       }
    }
 
@@ -434,10 +470,10 @@ gint motion_event(GtkWidget *w,GdkEventMotion *ev,gpointer dummy)
 
    if (ev->state & Button1Mask) {
       /* drag with button1: rotate */
-      theta-=180.*(double)dy/(double)winsizey/sqrt(zoom);
+      theta-=180.*(double)dy/(double)winsizey;
       if (theta<0) theta=0;
       if (theta>180) theta=180;
-      phi-=180.*(double)dx/(double)winsizex/sqrt(zoom);
+      phi-=180.*(double)dx/(double)winsizex;
       scaleplot=0;
       calc_vgain();
       if (plot2_vgain && redraw2==0) redraw2=2;
@@ -548,17 +584,26 @@ gint keypress_event(GtkWidget *w,GdkEventKey *ev,gpointer dummy)
          }
          switch (gainplot) {
             case GPnearfield: printf(" --near"); break;
-            case GP2d: printf(" --slice"); break;
-            case GP3d: printf(" --frame"); break;
+            case GPslice: printf(" --slice"); break;
+            case GPframe: printf(" --frame"); break;
+            case GPopaque: printf(" --opaque"); break;
          }
-         if (gainplot==GP2d || gainplot==GP3d) switch (gainscale) {
+         if (gainplot==GPslice || gainplot==GPframe || gainplot==GPopaque) switch (gainscale) {
             case GSlinpower: printf(" --linpower"); break;
             case GSlinvolt: printf(" --linvolt"); break;
             case GSlog: printf(" --log"); break;
             case GSarrl: printf(" --arrl"); break;
          }
+         if (gainplot==GPslice || gainplot==GPframe || gainplot==GPopaque || structplot==SPcurrents) switch (polarization) {
+            case POLnone: printf(" --pol=total"); break;
+            case POLhor: printf(" --pol=hor"); break;
+            case POLvert: printf(" --pol=vert"); break;
+            case POLlhcp: printf(" --pol=lhcp"); break;
+            case POLrhcp: printf(" --pol=rhcp"); break;
+            case POLcolour: printf(" --pol=colour"); break;
+         }
          printf("\n");
-         if (structplot!=SPnone) printf(" --view=%g,%g,%g,%g,%g",phi,theta,zoom,trx,try);
+         if (structplot!=SPnone || gainplot!=SPnone) printf(" --view=%g,%g,%g,%g,%g",phi,theta,zoom,trx,try);
          if (numneco>1) printf(" --freq=%g",neco[rp_index].f);
          printf("\n");
          if (structplot==SPanim || gainplot==GPnearfield) printf(" --afreq=%g --aphase=%g",animfreq,animphase*180/M_PI);
@@ -566,6 +611,11 @@ gint keypress_event(GtkWidget *w,GdkEventKey *ev,gpointer dummy)
          if (structplot==SPanim) printf(" --iscale=%g --qscale=%g",iscale, qscale);
          printf("\n");
          break;
+      case 'z':
+         if (animfreq==old_animfreq) break;
+         if (animfreq==0) { animfreq=old_animfreq; old_animfreq=0; }
+         else { old_animfreq=animfreq; animfreq=0; }
+         break;
       default:
          return FALSE;
    }
@@ -584,7 +634,7 @@ void cmd_reload(void)
    gtk_widget_grab_focus(gdraw2);
 }
 
-void cmd_X(Widget w,XtPointer client,XButtonEvent *ev)
+void cmd_X(void)
 {
    phi=0; theta=90; redraw=1;
    scaleplot=1;
@@ -595,7 +645,7 @@ void cmd_X(Widget w,XtPointer client,XButtonEvent *ev)
    gtk_widget_grab_focus(gdraw1);
 }
 
-void cmd_Y(Widget w,XtPointer client,XButtonEvent *ev)
+void cmd_Y()
 {
    phi=90; theta=90; redraw=1;
    scaleplot=1;
@@ -606,7 +656,7 @@ void cmd_Y(Widget w,XtPointer client,XButtonEvent *ev)
    gtk_widget_grab_focus(gdraw1);
 }
 
-void cmd_Z(Widget w,XtPointer client,XButtonEvent *ev)
+void cmd_Z()
 {
    phi=0; theta=0; redraw=1;
    scaleplot=1;
@@ -632,7 +682,7 @@ void cmd_setgs(char **p)
    gtk_widget_grab_focus(gdraw1);
 }
 
-char *GPnames[]={"none","slice","frame","near E/H",NULL};
+char *GPnames[]={"none","slice","frame","opaque","near E/H",NULL};
 
 void cmd_setgp(char **p)
 {
@@ -701,12 +751,19 @@ void cmd_togglelock(GtkWidget *w, gpointer *f)
 }
 
 
-char *POLnames[]={"ignore","horizontal","vertical","left circular","right circular",NULL};
+char *POLnames[]={"total","hor.","vert.","lhcp","rhcp","colour",NULL};
 
 void cmd_setpol(char **p)
 {
    polarization=p-POLnames;
+   mark_gpo_invalid();
+   if (rp_index>=0) { 
+      extr/=GAINSIZE; 
+      process_nec_output(neco+rp_index);
+   }
    redraw=1;
+   calc_vgain();
+   redraw2=1;
 }
 
 char *DCnames[]={"no","yes",NULL};
@@ -807,20 +864,19 @@ gint expose_event2(GtkWidget *w,GdkEventExpose *ev,gpointer dummy)
    if(ev->count != 0) return TRUE;
    if (rp_index<0) return TRUE;
    x=freqx(neco[rp_index].f);
-   XSetForeground(display,gc2,c_back.pixel);
-   XDrawLine(display,window2,gc2,x,0,x,win2sizey);
+   gdk_gc_set_foreground(ggc2,&c_back);
+   gdk_draw_line(gdraw2->window,ggc2,x,0,x,win2sizey);
    return TRUE;
 }
 
 
 gint resize_event2(GtkWidget *w,GdkEventConfigure *ev,gpointer dummy)
 {
-   if (!backg2) return FALSE;
+   if (!gbackg2) return FALSE;
    win2sizex=ev->width;
    win2sizey=ev->height;
    gdk_pixmap_unref(gbackg2);
-   gbackg2=gdk_pixmap_new(w->window,win2sizex,win2sizey,DefaultDepth(display,scrnr));
-   backg2=GDK_WINDOW_XWINDOW(gbackg2);
+   gbackg2=gdk_pixmap_new(w->window,win2sizex,win2sizey,depth);
 
    redraw2=1;
    return TRUE;
@@ -977,23 +1033,16 @@ void cmd_setZ0(GtkEntry *w)
 
 /* -------------------- X and GTK initialisation ------------------------------------------------ */
 
-void getcolor(char *name,XColor *xc)
+void getcolor(char *name,GdkColor *xc)
 {
-   XColor xe;
-   Colormap cm;
-
-   if (DefaultDepth(display,scrnr)==1) {     /* on 1bpp displays, choose black for everything except the background */
+   if (depth==1) {          /* on 1bpp displays, choose black for everything except the background */
       if (strcmp(name,C_BG)) name="black";
       else name="white";
    }
-
-   cm=DefaultColormap(display,scrnr);
-   if (!XAllocNamedColor(display,cm,name,xc,&xe)) {
+   gdk_color_parse(name,xc);
+   if (!gdk_colormap_alloc_color(gcm,xc,TRUE,FALSE)) {
       /* if allocation failed, use black */
-      xc->red=0;
-      xc->blue=0;
-      xc->green=0;
-      xc->pixel=BlackPixel(display,scrnr);
+      gdk_color_black(gcm,xc);
    }
 }
 
@@ -1087,9 +1136,7 @@ void get_currents_colors(void)
 {
    int i;
    double r,g,b;
-   Colormap cm;
 
-   cm=DefaultColormap(display,scrnr);
    for (i=0;i<NC_phase;i++) {
       r=i*360./NC_phase;
       g=0.5;
@@ -1098,7 +1145,7 @@ void get_currents_colors(void)
       c_currents[i].red=r*65535;
       c_currents[i].green=g*65535;
       c_currents[i].blue=b*65535;
-      XAllocColor(display,cm,c_currents+i);
+      gdk_colormap_alloc_color(gcm,c_currents+i,TRUE,TRUE);
    }
 }
 
@@ -1160,6 +1207,13 @@ void maininitX(int really)
    tt=gtk_tooltips_new();
 #define SET_TT(s) gtk_tooltips_set_tip(tt,w,s,NULL)
 
+   {
+      GdkVisual *vi;
+      vi = gdk_visual_get_best();
+      depth=vi->depth;
+      gcm=gdk_colormap_new(vi,FALSE);
+   }
+
    /* ----------- initialize window 1 (struct/gain plot) ------------------------------------------------------------- */
 
    if (window1open) {
@@ -1211,6 +1265,9 @@ void maininitX(int really)
       w=make_option_menu(GSnames,cmd_setgs,gainscale); gtk_box_pack_start(GTK_BOX(toprow), w, FALSE, FALSE, 0); gtk_widget_show(w);
       SET_TT("Choose gain scale");
 
+      w=make_option_menu(POLnames,cmd_setpol,polarization); gtk_box_pack_start(GTK_BOX(toprow), w, FALSE, FALSE, 0); gtk_widget_show(w);
+      SET_TT("Choose polarisation");
+
       w=gtk_button_new_with_label("X"); gtk_box_pack_start(GTK_BOX(toprow), w, FALSE, FALSE, 0); gtk_widget_show(w);
       gtk_signal_connect(GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(cmd_X), (gpointer)NULL);
       SET_TT("View along X axis");
@@ -1230,9 +1287,6 @@ void maininitX(int really)
 
       botrow_curr=gtk_hbox_new(FALSE,0);
 
-      w=make_option_menu(POLnames,cmd_setpol,polarization); gtk_box_pack_start(GTK_BOX(botrow_curr), w, FALSE, FALSE, 0); gtk_widget_show(w);
-      SET_TT("Choose polarisation");
-
       w=make_option_menu(DCnames,cmd_setdc,distcor); gtk_box_pack_start(GTK_BOX(botrow_curr), w, FALSE, FALSE, 0); gtk_widget_show(w);
       SET_TT("Toggle distance compensation");
 
@@ -1326,11 +1380,13 @@ void maininitX(int really)
 
       gtk_accel_group_attach(acc, GTK_OBJECT(win1));
 
-      display = GDK_WINDOW_XDISPLAY(w->window);
-      window = GDK_WINDOW_XWINDOW(w->window);
-      gc = XCreateGC(display, window, 0, NULL);
-      gbackg=gdk_pixmap_new(w->window,winsizex,winsizey,DefaultDepth(display,scrnr));
-      backg=GDK_WINDOW_XWINDOW(gbackg);
+      ggc = gdk_gc_new(w->window);
+
+      if (toprow->requisition.width>winsizex) {
+         winsizex=toprow->requisition.width;
+         calcproj();
+      }
+      gbackg=gdk_pixmap_new(w->window,winsizex,winsizey,depth);
 
       upd_msg();
 
@@ -1424,11 +1480,9 @@ void maininitX(int really)
 
       gtk_accel_group_attach(acc, GTK_OBJECT(win2));
 
-      display = GDK_WINDOW_XDISPLAY(w->window);
-      window2 = GDK_WINDOW_XWINDOW(w->window);
-      gc2 = XCreateGC(display, window2, 0, NULL);
-      gbackg2=gdk_pixmap_new(w->window,win2sizex,win2sizey,DefaultDepth(display,scrnr));
-      backg2=GDK_WINDOW_XWINDOW(gbackg2);
+      ggc2 = gdk_gc_new(w->window);
+
+      gbackg2=gdk_pixmap_new(w->window,win2sizex,win2sizey,depth);
 
       gtk_signal_connect( GTK_OBJECT(w), "expose_event", GTK_SIGNAL_FUNC(expose_event2), NULL);
 
@@ -1439,8 +1493,6 @@ void maininitX(int really)
 
    /* ----------- initialize some variables etc. that are needed for direct Xlib drawing ------------ */
 
-   scrnr = DefaultScreen(display);
-
    getcolor("white",&c_bg);
    getcolor(C_AXIS,&c_axis);
    getcolor(C_WIRE,&c_wire);
@@ -1457,19 +1509,18 @@ void maininitX(int really)
    getcolor(C_POYNTING,&c_poynting); 
    getcolor(C_QPOS,&c_qpos); 
    getcolor(C_QNEG,&c_qneg); 
+   getcolor(C_GAIN_LIN,&c_gain_lin);
+   getcolor(C_GAIN_LHCP,&c_gain_lhcp);
+   getcolor(C_GAIN_RHCP,&c_gain_rhcp);
    get_currents_colors(); 
   
-   fontinfo=XLoadQueryFont(display,XFONT);
-   if (fontinfo==NULL) fontinfo=XLoadQueryFont(display,"fixed");
-   if (fontinfo==NULL) { fprintf(stderr,"Can't load fonts '%s' or 'fixed'.\n",XFONT); exit(1); }
-   fontheight=fontinfo->ascent+fontinfo->descent;
+   gfont=gdk_font_load(XFONT);
+   if (!gfont) gfont=gdk_font_load("fixed");
+   if (!gfont) { fprintf(stderr,"Can't load fonts '%s' or 'fixed'.\n",XFONT); exit(1); }
+   fontheight=gfont->ascent+gfont->descent;
 
-   if (window1open) {
-      XSetFont(display,gc,fontinfo->fid);
-   }
-   if (window2open) {
-      XSetFont(display,gc2,fontinfo->fid);
-   }
+   if (window1open) gdk_gc_set_font(ggc,gfont);
+   if (window2open) gdk_gc_set_font(ggc2,gfont);
 
    if (gdraw1==NULL) gdraw1=gdraw2;
    else if (gdraw2==NULL) gdraw2=gdraw1;
@@ -1510,8 +1561,8 @@ void mainX(int really)
          if (rp_index>=0) {
             int x;
             x=freqx(neco[rp_index].f);
-            XSetForeground(display,gc2,c_back.pixel);
-            XDrawLine(display,window2,gc2,x,0,x,win2sizey);
+            gdk_gc_set_foreground(ggc2,&c_back);
+            gdk_draw_line(gdraw2->window,ggc2,x,0,x,win2sizey);
          }
       }
       animbusy=0;
